#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.


import os,copy,time, gobject
import gtk
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp
from classes import profiles, project, video
from windows import preferences
import AddEffect, TreeEffects

# init the foreign language
from language import Language_Init

class frmClipProperties(SimpleGtkBuilderApp):

	def __init__(self, path="ClipProperties.ui", root="frmClipProperties", domain="OpenShot", form=None, project=None, current_clip=None, current_clip_item=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		self._ = _

		self.form = form
		self.project = project
		self.current_clip = current_clip
		self.current_clip_item = current_clip_item
		self.restore_window_size()
		self.frmClipProperties.show_all()

		# Stop the video thread consumer
		self.form.MyVideo.pause()
		self.form.MyVideo.consumer_stop()
		# Force SDL to write on our drawing area
		os.putenv('SDL_WINDOWID', str(self.previewscreen.window.xid))
		gtk.gdk.flush()
		
		# make a copy of the current clip (for preview reasons)
		self.copy_of_clip = copy.copy(self.current_clip)
		self.copy_of_clip.keyframes = copy.deepcopy(self.current_clip.keyframes)
		self.copy_of_clip.effects = copy.deepcopy(self.current_clip.effects)
		self.copy_of_clip.position_on_track = 0

		# add items to direction combo
		options = ["16x", "15x", "14x", "13x", "12x", "11x", "10x", "9x", "8x", "7x", "6x", "5x", "4x", "3x", "2x", _("Normal Speed"), "1/2", "1/3", "1/4", "1/5", "1/6", "1/7", "1/8", "1/9", "1/10", "1/11", "1/12", "1/13", "1/14", "1/15", "1/16" ]
		# loop through export to options
		for option in options:
			# append profile to list
			self.cboSimpleSpeed.append_text(option)
			
		# add items to direction combo
		options = [_("Forward"), _("Reverse")]
		# loop through export to options
		for option in options:
			# append profile to list
			self.cboDirection.append_text(option)
			
		# add items to direction combo
		options = [_("Start of Clip"), _("End of Clip")]
		# loop through export to options
		for option in options:
			# append profile to list
			self.cboKeyFrame.append_text(option)
		
		# get file name
		(dirName, fname) = os.path.split(self.current_clip.file_object.name)
		
		# get a local keyframe collection
		self.keyframes = copy.deepcopy(self.current_clip.keyframes)
		self.current_keyframe = ""
		
		# init the default properties
		self.txtFileName.set_text(fname)
		self.txtLength.set_value(self.current_clip.length())
		self.txtIn.set_value(round(self.current_clip.start_time, 11))
		self.txtOut.set_value(round(self.current_clip.end_time, 11))
		self.txtAudioFadeAmount.set_value(round(self.current_clip.audio_fade_amount, 11))
		self.txtVideoFadeAmount.set_value(round(self.current_clip.video_fade_amount, 11))
		self.sliderVolume.set_value(self.current_clip.volume)
		self.chkFill.set_active(self.current_clip.fill)
		
		if self.current_clip.distort:
			self.chkMaintainAspect.set_active(False)
		else:
			self.chkMaintainAspect.set_active(True)
			
		self.chkVideoFadeIn.set_active(self.current_clip.video_fade_in)
		self.chkVideoFadeOut.set_active(self.current_clip.video_fade_out)
		self.chkAudioFadeIn.set_active(self.current_clip.audio_fade_in)
		self.chkAudioFadeOut.set_active(self.current_clip.audio_fade_out)
		
		self.spinAdvancedSpeed.set_value(self.current_clip.speed)
		self.set_horizontal_align_dropdown()
		self.set_vertical_align_dropdown()
		self.chkEnableVideo.set_active(self.current_clip.play_video)
		self.chkEnableAudio.set_active(self.current_clip.play_audio)
		self.set_direction_dropdown()
		self.set_keyframe_dropdown()
		self.set_speed_dropdown()
		
		# update effects tree
		self.OSTreeEffects = TreeEffects.OpenShotTree(self.treeEffects, self.project)
		self.update_effects_tree()

		# Refresh XML
		self.RefreshPreview()

		# start and stop the video
		self.form.MyVideo.play()
		self.form.MyVideo.pause()
		
		# mark project as modified
		# so that the real project is refreshed... even if these 
		# preview changes are not accepted
		self.project.set_project_modified(is_modified=False, refresh_xml=True)

		
	def restore_window_size(self):
		#default the form as not maximized
		self.is_maximized = False
		
		#load the window settings
		self.settings = preferences.Settings(self.project)
		self.settings.load_settings_from_xml()
			
		#get the app state settings
		self.width = int(self.settings.app_state["clip_property_window_width"])
		self.height = int(self.settings.app_state["clip_property_window_height"])
		is_max = self.settings.app_state["clip_property_window_maximized"]
		
		#resize window		
		self.frmClipProperties.resize(self.width, self.height)
		
		#maximize window if needed
		if is_max == "True":
			self.frmClipProperties.maximize()
		
		#set pane size from config xml
		self.hpaned1.set_position(int(self.settings.app_state["clip_property_hpane_position"]))
		
	def on_treeEffects_button_release_event(self, widget, *args):
		print "on_treeEffects_button_release_event"
		
		# get correct gettext method
		_ = self._
		
		# get selected effect (if any)
		selected_effect, unique_id = self.get_selected_effect()
		real_effect = self.OSTreeEffects.get_real_effect(service=selected_effect)
		clip_effect = self.get_clip_effect(unique_id)
		
		if real_effect:
			
			# Clear Effect Edit Controls
			self.clear_effect_controls()
			
			# Loop through Params
			param_index = 1
			for param in real_effect.params:
				# get hbox
				hbox = self.vboxcontainer.get_children()[param_index]
				label = hbox.get_children()[0]
				
				# Get actual value for param
				user_param_value = self.get_clip_parameter(clip_effect, param.name)
				
				# update label with title
				label.set_text(_(param.title))
				label.set_tooltip_text(_(param.title))

				if param.type == "spinner":
					# create spinner
					adj = gtk.Adjustment(float(user_param_value), float(param.min), float(param.max), 0.01, 0.01, 0.0)
					spinner = gtk.SpinButton(adj, 0.01, 2)
					# connect signal 
					spinner.connect("value-changed", self.effect_spinner_changed, real_effect, param, unique_id)
					# add to hbox
					hbox.pack_start(spinner, expand=True, fill=True)
					
				elif param.type == "hscale":
					# create hscale
					adj = gtk.Adjustment(float(user_param_value), float(param.min), float(param.max), 0.0, 0.0, 1.0)
					hscale = gtk.HScale(adj)
					# connect signal 
					hscale.connect("value-changed", self.effect_hscale_changed, real_effect, param, unique_id)
					# add to hbox
					hbox.pack_start(hscale, expand=True, fill=True)
				
				elif param.type == "dropdown":
					cboBox = gtk.combo_box_new_text()
					
					# add values
					box_index = 0
					for k,v in param.values.items():
						# add dropdown item
						cboBox.append_text(k)
						
						# select dropdown (if default)
						if v == user_param_value:
							cboBox.set_active(box_index)
						box_index = box_index + 1
						
					# connect signal
					cboBox.connect("changed", self.effect_dropdown_changed, real_effect, param, unique_id)
					# add to hbox
					hbox.pack_start(cboBox, expand=True, fill=True)
					
				elif param.type == "color":
					colorButton = gtk.ColorButton()
					# set color
					default_color = gtk.gdk.color_parse(user_param_value)
					colorButton.set_color(default_color)	

					# connect signal
					colorButton.connect("color-set", self.effect_color_changed, real_effect, param, unique_id)
					# add to hbox
					hbox.pack_start(colorButton, expand=True, fill=True)
				
				# show all new controls
				hbox.show_all()
				
				# increment param index
				param_index = param_index + 1
				
	def get_clip_effect(self, unique_id):
		""" find the effect object on the clip... that matches the service string. """
		# Loop through all effects
		for clip_effect in self.copy_of_clip.effects:
			if clip_effect.unique_id == unique_id:
				# Found clip effect... now we need to find the actual param to update
				return clip_effect
			
	def get_clip_parameter(self, clip_effect, parameter_name):
		""" Get the actual values that the user has saved for a clip effect paramater """
		for clip_param in clip_effect.paramaters:
			# find the matching param
			if parameter_name in clip_param.keys():
				# update the param
				return clip_param[parameter_name]
				
				
	def effect_spinner_changed(self, widget, real_effect, param, unique_id, *args):
		print "effect_spinner_changed"
		
		# Update the param of the selected effect
		for clip_effect in self.copy_of_clip.effects:
			if clip_effect.unique_id == unique_id:
				# Found clip effect... now we need to find the actual param to update
				for clip_param in clip_effect.paramaters:
					# find the matching param
					if param.name in clip_param.keys():
						# update the param
						clip_param[param.name] = str(widget.get_value())
						return
					
	def effect_hscale_changed(self, widget, real_effect, param, unique_id, *args):
		print "effect_hscale_changed"
		
		# Update the param of the selected effect
		for clip_effect in self.copy_of_clip.effects:
			if clip_effect.unique_id == unique_id:
				# Found clip effect... now we need to find the actual param to update
				for clip_param in clip_effect.paramaters:
					# find the matching param
					if param.name in clip_param.keys():
						# update the param
						clip_param[param.name] = str(widget.get_value())
						return
		
	def effect_dropdown_changed(self, widget, real_effect, param, unique_id, *args):
		print "effect_dropdown_changed"
		
		# find numeric value of dropdown selection
		dropdown_value = ""
		for k,v in param.values.items():		
			if k == widget.get_active_text():
				dropdown_value = v
		
		# Update the param of the selected effect
		for clip_effect in self.copy_of_clip.effects:
			if clip_effect.unique_id == unique_id:
				# Found clip effect... now we need to find the actual param to update
				for clip_param in clip_effect.paramaters:
					# find the matching param
					if param.name in clip_param.keys():
						# update the param
						clip_param[param.name] = dropdown_value
						return
		
	def effect_color_changed(self, widget, real_effect, param, unique_id, *args):
		print "effect_color_changed"
		
		# Get color from color picker
		color = widget.get_color()

		# Update the param of the selected effect
		for clip_effect in self.copy_of_clip.effects:
			if clip_effect.unique_id == unique_id:
				# Found clip effect... now we need to find the actual param to update
				for clip_param in clip_effect.paramaters:
					# find the matching param
					if param.name in clip_param.keys():
						# update the param
						clip_param[param.name] = self.html_color(color)
						return
					
	def html_color(self, color):
		'''converts the gtk color into html color code format'''
		return '#%02x%02x%02x' % (color.red/256, color.green/256, color.blue/256)
			
	def clear_effect_controls(self):
		
		# Loop through all child hboxes
		child_index = 0
		for hbox in self.vboxcontainer.get_children():
			
			if child_index > 0: 
				# get Label
				label = hbox.get_children()[0]
				label.set_text("")
				
				# remove input control (if any)
				if len(hbox.get_children()) > 1:
					# remove the item
					hbox.remove(hbox.get_children()[1])
					
			# increment child index
			child_index = child_index + 1
	
	def update_effects_tree(self):
		
		# Populate effects tree again
		self.OSTreeEffects.populate_tree(self.copy_of_clip.effects)
		

	def get_selected_effect(self):
		# Get Effect service name
		selection = self.treeEffects.get_selection()
		rows, selected = selection.get_selected_rows()
		iters = [rows.get_iter(path) for path in selected]
		for iter in iters:
			Name_of_Effect = self.treeEffects.get_model().get_value(iter, 1)
			Effect_Service = self.treeEffects.get_model().get_value(iter, 2)
			unique_id = self.treeEffects.get_model().get_value(iter, 3)
			return Effect_Service, unique_id
		
		# no selected item
		return None, None
	
		
	def on_frmClipProperties_window_state_event(self, widget, event, *args):
		# determine if properties window is maximized or not 
		
		if event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED:
			if event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
				# maximized
				self.is_maximized = True
			else:
				# not maximized
				self.is_maximized = False
				
		# refresh sdl on window resize
		if self.form.MyVideo:
			self.form.MyVideo.refresh_sdl()

		
	def on_frmClipProperties_destroy(self, widget, *args):
		#print "on_frmClipProperties_destroy"
		#get the property window size
		self.settings.app_state["clip_property_window_maximized"] = str(self.is_maximized)
		self.settings.app_state["clip_property_window_width"] = self.width
		self.settings.app_state["clip_property_window_height"] = self.height
		self.settings.app_state["clip_property_hpane_position"] = self.hpaned1.get_position()
		
		#save the settings
		self.settings.save_settings_to_xml()
		
		
	def on_frmClipProperties_close(self, widget, *args):
		print "on_frmClipProperties_close"
		
		# close the window
		self.close_window()
		
		return True
	
	def on_frmClipProperties_configure_event(self, widget, *args):
		print "on_frmClipProperties_configure_event"
		
		#handles the resize event of the window
		#set the new width and height from the window resize
		(self.width, self.height) = self.frmClipProperties.get_size()	

		# refresh sdl on window resize
		if self.form.MyVideo:
			self.form.MyVideo.refresh_sdl()
		
		
	def on_btnAddEffect_clicked(self, widget, *args):
		#print "on_btnAddEffect_clicked"
		
		# show frmExportVideo dialog
		self.frmAddEffect = AddEffect.frmAddEffect(parent=self, form=self.form, project=self.project)
		
		# clear effect controls
		self.clear_effect_controls()
		

	def on_btnRemoveEffect_clicked(self, widget, *args):
		print "on_btnRemoveEffect_clicked"
		
		# Get selected effect
		selected_service, unique_id = self.get_selected_effect()
		if selected_service:
			# Remove effect
			self.copy_of_clip.Remove_Effect(unique_id)
		
			# update effect tree
			self.update_effects_tree()
			
			# clear effect controls
			self.clear_effect_controls()
		
		
	def on_btnEffectUp_clicked(self, widget, *args):
		print "on_btnEffectUp_clicked"
		
		# Get selected effect
		selected_service, unique_id = self.get_selected_effect()
		if selected_service:
			# Move Effect
			self.copy_of_clip.Move_Effect(unique_id, "up")
			
			# update effect tree
			self.update_effects_tree()
			
			# clear effect controls
			self.clear_effect_controls()
	
	def on_btnEffectDown_clicked(self, widget, *args):
		print "on_btnEffectDown_clicked"
		
		# Get selected effect
		selected_service, unique_id = self.get_selected_effect()
		if selected_service:
			# Move Effect
			self.copy_of_clip.Move_Effect(unique_id, "down")
			
			# update effect tree
			self.update_effects_tree()
			
			# clear effect controls
			self.clear_effect_controls()

	def on_btnResetVolume_clicked(self, widget, *args):
		print "on_btnResetVolume_clicked"
		
		# reset the speed to 1.0
		self.sliderVolume.set_value(100)		

	def on_txtHeight_value_changed(self, widget, *args):
		print "on_txtHeight_value_changed"
		
		# update property
		self.keyframes[self.current_keyframe].height = self.txtHeight.get_value()
		
	def on_txtWidth_value_changed(self, widget, *args):
		print "on_txtWidth_value_changed"
		
		# update property
		self.keyframes[self.current_keyframe].width = self.txtWidth.get_value()
		
	def on_txtX_value_changed(self, widget, *args):
		print "on_txtX_value_changed"
		
		# update property
		self.keyframes[self.current_keyframe].x = self.txtX.get_value()
		
	def on_txtY_value_changed(self, widget, *args):
		print "on_txtY_value_changed"
		
		# update property
		self.keyframes[self.current_keyframe].y = self.txtY.get_value()
		
	def on_scaleAlpha_value_changed(self, widget, *args):
		print "on_scaleAlpha_value_changed"
		
		# update property
		self.keyframes[self.current_keyframe].alpha = float(self.scaleAlpha.get_value()) / 100.0
		

	def on_cboKeyFrame_changed(self, widget, *args):
		
		# get correct gettext method
		_ = self._
		
		print "on_cboKeyFrame_changed"
		localcboKeyFrame = self.cboKeyFrame.get_active_text().lower()
		keyframe = None
		
		if localcboKeyFrame == _("Start of Clip").lower():
			# get start keyframe
			self.current_keyframe = "start"
			keyframe = self.keyframes[self.current_keyframe]
		else:
			# get end keyframe
			self.current_keyframe = "end"
			keyframe = self.keyframes[self.current_keyframe]

		# update keyframe widgets
		self.txtHeight.set_value(float(keyframe.height))
		self.txtWidth.set_value(float(keyframe.width))
		self.txtX.set_value(float(keyframe.x))
		self.txtY.set_value(float(keyframe.y))
		self.scaleAlpha.set_value(keyframe.alpha * 100)

		
	def on_cboSimpleSpeed_changed(self, widget, *args):
		
		# get correct gettext method
		_ = self._
		
		print "on_cboSimpleSpeed_changed"	
		localcboSimpleSpeed = self.cboSimpleSpeed.get_active_text().lower().replace("x","")
		
		if localcboSimpleSpeed == _("Normal Speed").lower():
			num = 1.0
			den = 1.0
		else:
			# calculate decimal speed
			arrSpeed = localcboSimpleSpeed.split("/")
			if len(arrSpeed) == 1:
				num = float(arrSpeed[0])
				den = 1.0
			else:
				num = float(arrSpeed[0])
				den = float(arrSpeed[1])
				
		# set the advanced speed textbox
		self.spinAdvancedSpeed.set_value(num / den)
		
	def on_txtIn_value_changed(self, widget, *args):
		print "on_txtIn_value_changed"
		local_in = float(self.txtIn.get_value())
		local_out = float(self.txtOut.get_value())
		
		# is IN valid?
		if local_in >= local_out:
			local_in = local_out - 0.000000001
			self.txtIn.set_text(str(local_in))
		
		# update length
		self.txtLength.set_text(str(round(local_out - local_in, 11)))
		
	def on_txtOut_value_changed(self, widget, *args):
		print "on_txtOut_value_changed"
		local_in = float(self.txtIn.get_value())
		local_out = float(self.txtOut.get_value())
		local_max_length = self.current_clip.max_length
		
		# is IN valid?
		if local_out <= local_in:
			local_out = local_in + 0.000000001
			self.txtOut.set_text(str(local_out))
		
		if local_out > local_max_length:
			local_out = local_max_length
			self.txtOut.set_text(str(local_out))
		
		# update length
		self.txtLength.set_text(str(round(local_out - local_in, 11)))

	def set_keyframe_dropdown(self):
		# get the model and iterator of the project type dropdown box
		model = self.cboKeyFrame.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)

			# set the item as active
			self.cboKeyFrame.set_active_iter(iter)
			break

			
			
	def set_speed_dropdown(self):
		
		# get correct gettext method
		_ = self._
		
		# get the model and iterator of the project type dropdown box
		model = self.cboSimpleSpeed.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0).lower().replace("x", "")
			
			if value == _("Normal Speed").lower():
				num = 1.0
				den = 1.0
			else:
				# calculate decimal speed
				arrSpeed = value.split("/")
				if len(arrSpeed) == 1:
					num = float(arrSpeed[0])
					den = 1.0
				else:
					num = float(arrSpeed[0])
					den = float(arrSpeed[1])

				
			# check for the matching project type
			if self.current_clip.speed >= 1.0 and self.current_clip.speed == num:
				
				# set the item as active
				self.cboSimpleSpeed.set_active_iter(iter)
				break
			
			# check for the matching project type
			if self.current_clip.speed < 1.0 and self.current_clip.speed == num / den:
				
				# set the item as active
				self.cboSimpleSpeed.set_active_iter(iter)
				break
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				# collapse simple, expand advanced
				self.expanderSimple.set_expanded(False)
				self.expanderAdvanced.set_expanded(True)
				break
			
				
			
			


	def on_btnResetSpeed_clicked(self, widget, *args):
		print "on_btnResetSpeed_clicked"
		
		# reset the speed to 1.0
		self.sliderSpeed.set_value(1.0)

		
	def set_direction_dropdown(self):
		
		# get correct gettext method
		_ = self._
		
		# get the model and iterator of the project type dropdown box
		model = self.cboDirection.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)

			# check for the matching project type
			if self.current_clip.reversed == True and value.lower() == _("Reverse").lower():			
				
				# set the item as active
				self.cboDirection.set_active_iter(iter)
				
			# check for the matching project type
			if self.current_clip.reversed == False and value.lower() == _("Forward").lower():			
				
				# set the item as active
				self.cboDirection.set_active_iter(iter)
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
		


	def set_horizontal_align_dropdown(self):
		# get the model and iterator of the project type dropdown box
		model = self.cboHAlign.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)

			# check for the matching project type
			if self.current_clip.halign.lower() == value.lower():			
				
				# set the item as active
				self.cboHAlign.set_active_iter(iter)
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
	def set_vertical_align_dropdown(self):
		# get the model and iterator of the project type dropdown box
		model = self.cboVAlign.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)

			# check for the matching project type
			if self.current_clip.valign.lower() == value.lower():			
				
				# set the item as active
				self.cboVAlign.set_active_iter(iter)
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
	def apply_settings(self, clip_object):
		
		# get correct gettext method
		_ = self._
		
		# get the frames per second (from the project)
		fps = self.project.fps()
		
		# Get settings
		localcboEnableVideo = self.chkEnableVideo.get_active()
		localcboEnableAudio = self.chkEnableAudio.get_active()
		localtxtIn = self.txtIn.get_value()
		localtxtOut = self.txtOut.get_value()
		localtxtLength = self.txtLength.get_value()
		localcboFill = self.chkFill.get_active()
		localchkMaintainAspect = self.chkMaintainAspect.get_active()
		localcboDirection = self.cboDirection.get_active_text().lower()
		localsliderSpeed = self.spinAdvancedSpeed.get_value()
		localtxtAudioFadeAmount = self.txtAudioFadeAmount.get_value()
		localtxtVideoFadeAmount = self.txtVideoFadeAmount.get_value()
		localsliderVolume = self.sliderVolume.get_value()

		# reset the IN, OUT textboxes to original speed
		original_speed = clip_object.get_speed()
		localtxtIn = localtxtIn * original_speed
		localtxtOut = localtxtOut * original_speed
		localtxtLength = localtxtLength * original_speed

		localtxtHeight = self.txtHeight.get_value()
		localtxtWidth = self.txtWidth.get_value()
		localtxtX = self.txtX.get_value()
		localtxtY = self.txtY.get_value()
		localcboHAlign = self.cboHAlign.get_active_text().lower()
		localcboVAlign = self.cboVAlign.get_active_text().lower()
		localscaleAlpha = self.scaleAlpha.get_value()
		
		# update clip object
		clip_object.play_video = localcboEnableVideo
		clip_object.play_audio = localcboEnableAudio
		clip_object.speed = localsliderSpeed	# set new speed
		clip_object.halign = localcboHAlign
		clip_object.valign = localcboVAlign
		clip_object.audio_fade_amount = localtxtAudioFadeAmount
		clip_object.video_fade_amount = localtxtVideoFadeAmount
		clip_object.volume = localsliderVolume
		
		clip_object.video_fade_in = self.chkVideoFadeIn.get_active()
		clip_object.video_fade_out = self.chkVideoFadeOut.get_active()
		clip_object.audio_fade_in = self.chkAudioFadeIn.get_active()
		clip_object.audio_fade_out = self.chkAudioFadeOut.get_active()
		
		clip_object.fill = localcboFill
		if localchkMaintainAspect:
			clip_object.distort = False
		else:
			clip_object.distort = True
		
		# get new speed of clip (and update end_time... and thus length)
		speed_multiplier = clip_object.get_speed()
		original_length = clip_object.length()
		clip_object.max_length = clip_object.file_object.length / speed_multiplier
		clip_object.start_time = localtxtIn / speed_multiplier
		new_end_time = localtxtOut / speed_multiplier
		
		if speed_multiplier < original_speed:
			# clip is longer now (keep the short version)
			clip_object.end_time = clip_object.start_time + original_length
		else:
			# clip is shorter
			clip_object.end_time = new_end_time
		
		# update keyframes
		clip_object.keyframes = self.keyframes
		
		if localcboDirection.lower() == _("Reverse").lower():
			clip_object.reversed = True
		else:
			clip_object.reversed = False

	def on_btnPlay_clicked(self, widget, *args):
		print "on_btnPlay_clicked"

		# Get the current speed
		current_speed = self.form.MyVideo.get_speed()
		position = self.form.MyVideo.position()
		
		# Refresh Preview XML
		self.RefreshPreview()
		
		# seek back to position
		self.form.MyVideo.seek(position)

		# is video stopped?
		if current_speed == 0:
			# start video
			self.form.MyVideo.play()
		else:
			# stop video
			self.form.MyVideo.pause()
			

		
		
	def on_hpaned1_size_allocate(self, widget, *args):
		#print "on_hpanel1_size_allocate"
		
		if self.form.MyVideo:
			# refresh sdl
			self.form.MyVideo.refresh_sdl()
			

	def on_hsPreviewProgress_button_press_event(self, widget, *args):
		#print "on_hsPreviewProgress_button_press_event"
		
		# get the percentage of the video progress 0 to 100
		video_progress_percent = float(self.hsPreviewProgress.get_value()) / 100.0

		# determine frame number
		new_frame = int(float(self.form.MyVideo.get_length() - 1) * video_progress_percent)

		# Refresh Preview XML
		self.RefreshPreview()

		# jump to this frame
		self.form.MyVideo.seek(new_frame)


	def on_hsPreviewProgress_change_value(self, widget, *args):
		#print "on_hsPreviewProgress_value_changed"

		# get the percentage of the video progress 0 to 100
		video_progress_percent = float(self.hsPreviewProgress.get_value()) / 100.0

		# determine frame number
		new_frame = int(float(self.form.MyVideo.get_length() - 1) * video_progress_percent)

		# jump to this frame
		self.form.MyVideo.seek(new_frame)
		
	def RefreshPreview(self):
		# Apply settings to preview clip object
		self.apply_settings(self.copy_of_clip)

		# Generate the Preview XML
		self.copy_of_clip.GeneratePreviewXML(os.path.join(self.project.USER_DIR, "preview.mlt"), "clip-properties-preview")

		# Hook up video thread to new alternative progress bar
		#self.form.MyVideo.alternate_progress_bar = self.hsPreviewProgress
		gobject.idle_add(self.form.MyVideo.set_progress_bar, self.hsPreviewProgress)

		# Load XML
		self.form.MyVideo.set_project(self.project, self.form, os.path.join(self.project.USER_DIR, "preview.mlt"), mode="preview")
		self.form.MyVideo.load_xml()


	def on_btnCancel_clicked(self, widget, *args):
		print "on_btnCancel_clicked"
		
		# close the window
		self.close_window()


	def on_btnClose_clicked(self, widget, *args):
		print "on_btnClose_clicked"
		
		# only close the window if it finds a current_clip object
		if self.current_clip:
		
			# Apply settings to current clip object
			self.apply_settings(self.current_clip)
			
			# update the effects list
			self.current_clip.effects = copy.deepcopy(self.copy_of_clip.effects)
			
			# mark project as modified
			self.project.set_project_modified(is_modified=True, refresh_xml=True, type = self._("Modified clip properties"))
			
			# remove from canvas
			parent = self.current_clip_item.get_parent()
			child_num = parent.find_child (self.current_clip_item)
			parent.remove_child (child_num)
			
			# re-render just this clip
			self.current_clip.RenderClip()
			
			# raise the play-head
			self.project.sequences[0].raise_play_head()
			
			# check if the timeline needs to be expanded
			self.form.expand_timeline(self.current_clip)
			
			# clear the clip object
			self.current_clip = None
			
			# close the window
			self.close_window()
		
		
	def close_window(self):
		""" Have the video thread close this window, to prevent SDL crashes """
		# Force SDL to write on our drawing area
		os.putenv('SDL_WINDOWID', str(self.form.videoscreen.window.xid))
		gtk.gdk.flush()
		
		# Stop the video thread consumer
		self.form.MyVideo.pause()
		self.form.MyVideo.consumer_stop()
		
		# Un-hook alternative progress bar
		gobject.idle_add(self.form.MyVideo.clear_progress_bar)
		gobject.idle_add(self.form.MyVideo.close_window, self.frmClipProperties)
	
			
def main():
	frmClipProperties = frmClipProperties()
	frmClipProperties.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#   Copyright (C) 2009  TJ, Jonathan Thomas
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

VERSION             = "1.2.2"
DATE                = "20100919170000"
NAME                = 'openshot'
GPL_VERSION         = '3'
SUPPORTED_LANGUAGES = ['English', 'Dutch', 'French', 'German', 'Italian', 'Portuguese', 'Spanish', 'Swedish']

# names of all contributers, using 'u' for unicode encoding
AF   = {'name': u'Andy Finch', 'email': 'we.rocked.in79@gmail.com'}
HMC  = {'name': u'Helen McCall', 'email': 'wildnfree@blueyonder.co.uk'}
JT   = {'name': u'Jonathan Thomas', 'email': 'jonathan.oomph@gmail.com'}
OG   = {'name': u'Olivier Girard', 'email': 'eolinwen@gmail.com'}
TJ   = {'name': u'TJ', 'email': 'ubuntu@tjworld.net'}
MA   = {'name': u'Mael Lavault', 'email': 'moimael@neuf.fr'}
IM   = {'name': u'itomailg', 'email': 'itomailg@gmail.com'}


#credits
CREDITS = {
    'code'          : [JT, AF, TJ, OG, MA, IM],
    'documentation' : [OG, JT, HMC],
    'translation'   : [HMC, OG],
}

SETUP   = {
    'name'              : NAME,
    'version'           : VERSION,
    'author'            : JT['name'] + ' and others',
    'author_email'      : JT['email'],
    'maintainer'        : JT['name'],
    'maintainer_email'  : JT['email'],
    'url'               : 'http://www.openshotvideo.com/',
    'license'           : 'GNU GPL v.' + GPL_VERSION,
	'description'	   : 'Create and edit videos and movies',
	'long_description'  : "Create and edit videos and movies\n OpenShot Video Editor is a free, open-source, non-linear video editor.  It\n can create and edit videos and movies using many popular video, audio, and\n image formats.  Create videos for YouTube, Flickr, Vimeo, Metacafe, iPod,\n Xbox, and many more common formats!\n .\n Features include:\n  * Multiple tracks (layers)\n  * Compositing, image overlays, and watermarks\n  * Support for image sequences (rotoscoping)\n  * Key-frame animation\n  * Video and audio effects (chroma-key)\n  * Transitions (lumas and masks)",
		 

# see http://pypi.python.org/pypi?%3Aaction=list_classifiers
    'classifiers'       : [          
                           'Development Status :: 4 - Beta',
                           'Environment :: X11 Applications',
                           'Environment :: X11 Applications :: GTK',
                           'Intended Audience :: End Users/Desktop',
                           'License :: OSI Approved :: GNU General Public License (GPL)',
                           'Operating System :: OS Independent',
                           'Operating System :: POSIX :: Linux',
                           'Programming Language :: Python',
                           'Topic :: Artistic Software',
                           'Topic :: Multimedia :: Video :: Non-Linear Editor',] +
                          ['Natural Language :: ' + language for language in SUPPORTED_LANGUAGES
                          ],
}




#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#  
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.
#
# Profles Manager  Copyright (C) 2009  Andy Finch
#

import os, sys
import gtk
import tarfile
import shutil
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp
from windows import preferences
from classes import profiles, messagebox


# init the foriegn language
import language.Language_Init as Language_Init

class frmProfilesManager(SimpleGtkBuilderApp):

	def __init__(self, path="profiles.ui", root="frmProfiles", domain="OpenShot", form=None, parent=None, project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		self._ = _

		self.project = project
		self.form = form
		self.parent = parent

		#find path where openshot is running
		self.path = self.project.BASE_DIR
		
		# init the list of possible project types / profiles
		self.profile_list = profiles.mlt_profiles(self.project).get_profile_list()
		
		# loop through each profile, and add it to the dropdown
		for file_name, p in self.profile_list:
			# append profile to list
			self.cmbProjectType.append_text(p.description())
			
		self.save_prompt = False
		
		image = gtk.Image()
		image.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON)
		self.btnImport.set_image(image)
			
		self.frmProfiles.show_all()
		
		#set the default profile
		self.set_project_type_dropdown(self.form.settings.general["default_profile"])
		if not self.cmbProjectType.get_active_text():
			self.set_project_type_dropdown("DV NTSC")
		
	
	def get_profiles_list(self):
		
		# init the list of possible project types / profiles
		self.profile_list = profiles.mlt_profiles(self.project).get_profile_list()
		
		# loop through each profile, and add it to the dropdown
		for file_name, p in self.profile_list:
			# append profile to list
			self.cmbProjectType.append_text(p.description())
		
	def set_project_type_dropdown(self, item):
		
		# get reference to gettext
		_ = self._
		
		# get the model and iterator of the project type dropdown box
		model = self.cmbProjectType.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)
			
			# check for the matching project type
			if item == value:			
				
				# set the item as active
				self.cmbProjectType.set_active_iter(iter)
				self.init_properties(value)
				break
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
				
		
	def on_cmbProjectType_changed(self, widget, *args):
		
		# init the project type properties
		self.init_properties(self.cmbProjectType.get_active_text())
		
		
	def init_properties(self, profile):
		
		# get correct gettext method
		_ = self._
		
		# get the mlt profile
		localType = profile 
		p = profiles.mlt_profiles(self.project).get_profile(localType)

		# populate the labels with values
		self.spnSizeHeight.set_text(str(p.height()))
		self.spnSizeWidth.set_text(str(p.width()))
		self.spnAspect1.set_text(str(p.display_aspect_num()))
		self.spnAspect2.set_text(str(p.display_aspect_den()))
		self.spnFrameRate1.set_text(str(p.frame_rate_num()))
		self.spnFrameRate2.set_text(str(p.frame_rate_den()))
		self.spnPixelRatio1.set_text(str(p.sample_aspect_num()))
		self.spnPixelRatio2.set_text(str(p.sample_aspect_den()))
			
		if p.progressive():
			self.chkProgressive.set_active(True)
		else:
			self.chkProgressive.set_active(False)
			
			
	def on_btnNew_clicked(self, widget, *args):
		#activate the spinbuttons etc.
		self.spnSizeHeight.set_sensitive(True)
		self.spnSizeWidth.set_sensitive(True)
		self.lblSize.set_sensitive(True)
		self.spnAspect1.set_sensitive(True)
		self.spnAspect2.set_sensitive(True)
		self.lblAspect.set_sensitive(True)
		self.spnPixelRatio1.set_sensitive(True)
		self.spnPixelRatio2.set_sensitive(True)
		self.lblPixel.set_sensitive(True)
		self.chkProgressive.set_sensitive(True)
		self.chkDefaultProfile.set_sensitive(True)
		self.txtProfileName.set_sensitive(True)
		self.lblProfile.set_sensitive(True)
		self.spnFrameRate1.set_sensitive(True)
		self.spnFrameRate2.set_sensitive(True)
		self.lblFrame.set_sensitive(True)
		self.btnSave.set_sensitive(True)
		
		#disable the drop down
		self.cmbProjectType.set_sensitive(False)
		
		self.txtProfileName.grab_focus()
		
		self.save_prompt = True
		
		
	def on_btnSave_clicked(self, widget, *args):
		
		file_name = self.txtProfileName.get_text()
		
		#first check if this name already exists in the 'preset' profiles
		#or USER_PROFILES_DIR
		if profiles.mlt_profiles(self.project).profile_exists(file_name):
			messagebox.show("Openshot", _("A profile with this name already exists. Please use another name."))
			return True
		else:
			self.save_profile()
		
		
	def save_profile(self):
		
		#check we have all the data populated
		if self.txtProfileName.get_text() == "":
			messagebox.show("Openshot Error!", _("Please enter a profile name."))
			self.txtProfileName.grab_focus()
			return
			
		if self.spnSizeHeight.get_text() == "0" or self.spnSizeWidth == "0":
			messagebox.show("Openshot Error!", _("Please enter a valid size."))
			return
		
		####create the profile file####
		filename = self.txtProfileName.get_text()
		if self.chkProgressive.get_active():
			progressive = '1'
		else:
			progressive = '0'
		f=open(os.path.join(self.project.USER_PROFILES_DIR, filename),'w')
		f.write("description=" + filename + '\n')
		f.write("frame_rate_num=" + self.spnFrameRate1.get_text() + '\n')
		f.write("frame_rate_den=" + self.spnFrameRate2.get_text() + '\n')
		f.write("width=" + self.spnSizeWidth.get_text() + '\n')
		f.write("height=" + self.spnSizeHeight.get_text() + '\n')
		f.write("progressive=" + progressive + '\n')
		f.write("sample_aspect_num=" + self.spnPixelRatio1.get_text() + '\n')
		f.write("sample_aspect_den=" + self.spnPixelRatio2.get_text() + '\n')
		f.write("display_aspect_num=" + self.spnAspect1.get_text() + '\n')
		f.write("display_aspect_den=" + self.spnAspect2.get_text() + '\n')
		f.close()
		
		# reload parent window's list of profiles
		self.parent.populate_profiles()
		
		#check for default profile
		if self.chkDefaultProfile.get_active():
			# change default profile
			self.parent.set_default_profile(filename)
		else:
			# re-select the default profile
			self.parent.set_default_profile(self.form.settings.general["default_profile"])
		
		#reactivate the drop down
		self.cmbProjectType.set_sensitive(False)
		
		self.btnSave.set_sensitive(False)
		
		#disable the spinbuttons
		self.spnSizeHeight.set_sensitive(False)
		self.spnSizeWidth.set_sensitive(False)
		self.lblSize.set_sensitive(False)
		self.spnAspect1.set_sensitive(False)
		self.spnAspect2.set_sensitive(False)
		self.lblAspect.set_sensitive(False)
		self.spnPixelRatio1.set_sensitive(False)
		self.spnPixelRatio2.set_sensitive(False)
		self.lblPixel.set_sensitive(False)
		self.chkProgressive.set_sensitive(False)
		self.chkDefaultProfile.set_sensitive(False)
		self.txtProfileName.set_sensitive(False)
		self.lblProfile.set_sensitive(False)
		self.spnFrameRate1.set_sensitive(False)
		self.spnFrameRate2.set_sensitive(False)
		self.lblFrame.set_sensitive(False)
		self.btnSave.set_sensitive(False)
		
		#enable the drop down
		self.cmbProjectType.set_sensitive(True)
		
		#add the newly saved profile to the combo
		self.cmbProjectType.append_text(filename)
		self.set_project_type_dropdown(filename)
		
		self.save_prompt = False
		
	def on_btnClose_clicked(self, widget, *args):
		
		if self.save_prompt == True:
			messagebox.show("Openshot", _("Do you want to save this profile?."), gtk.BUTTONS_YES_NO, self.save_profile, self.close_window)
		else:
			self.close_window()
			
	def on_btnDelete_clicked(self, widget, *args):
		
		# get correct gettext method
		_ = self._

		#get the active profile
		model = self.cmbProjectType.get_model()
		iter = self.cmbProjectType.get_active_iter()
		file_name = model.get_value(iter, 0)
		
		messagebox.show("Openshot", _("Are you sure you want to delete the profile %s?" % file_name), gtk.BUTTONS_YES_NO, self.delete_profile)
		
		
			
	def close_window(self):
		
		self.frmProfiles.destroy()
		
	def delete_profile(self):
		
		#get the active profile
		model = self.cmbProjectType.get_model()
		iter = self.cmbProjectType.get_active_iter()
		file_name = model.get_value(iter, 0)

		if os.path.exists(os.path.join(self.project.USER_PROFILES_DIR, file_name)):
			os.remove(os.path.join(self.project.USER_PROFILES_DIR, file_name))
			#clear the combo & repopulate
			model.clear()	
			self.get_profiles_list()
			
			#set the default profile
			self.set_project_type_dropdown(self.form.settings.general["default_profile"])
			if not self.cmbProjectType.get_active_text():
				self.set_project_type_dropdown("DV NTSC")
				self.parent.set_default_profile("DV NTSC")

			# reload parent window's list of profiles
			self.parent.populate_profiles()
			
			# re-select the default profile
			self.parent.set_default_profile(self.form.settings.general["default_profile"])
			
		else:
			messagebox.show("Openshot Error!", _("The profile %s cannot be deleted." % file_name))
	
			
	def on_btnImport_clicked(self, widget, *args):
				
		file_open = gtk.FileChooserDialog(title="Select Profile"
					, action=gtk.FILE_CHOOSER_ACTION_OPEN
					, buttons=(gtk.STOCK_CANCEL
								, gtk.RESPONSE_CANCEL
								, gtk.STOCK_OPEN
								, gtk.RESPONSE_OK))
		#add the file filters
		filter = gtk.FileFilter()
		filter.set_name("tar archives")
		filter.add_pattern("*.tar.gz")
		filter.add_pattern("*.tar.bz2")
		file_open.add_filter(filter)
		#add the 'all files' filter"""
		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		file_open.add_filter(filter)
		
		if file_open.run() == gtk.RESPONSE_OK:
			file_name = file_open.get_filename()
			
			(dirName, filename) = os.path.split(file_name)
			(fileBaseName, fileExtension)=os.path.splitext(filename)
		
			if ".bz2" in fileExtension or ".gz" in fileExtension:
				#extract the file(s)
				try:
					tar = tarfile.open(file_name)
					tar.extractall(self.project.USER_PROFILES_DIR)
					tar.close()
					messagebox.show("Openshot", _("The profile has been successfully imported."))
				except:
					messagebox.show("Openshot Error!", _("There was an error extracting the file %s" % filename))
					
				
				
			else:
				#regular file - just copy it to the user profiles folder
				# get file name, path, and extention
				
				try:
					shutil.copyfile(file_name, os.path.join(self.project.USER_PROFILES_DIR, fileBaseName))
					messagebox.show("Openshot", _("The profile has been successfully imported."))
				except:
					messagebox.show("Openshot Error!", _("There was an error copying the file %s" % filename))	
					
			#add the new profile to the dropdown
			#clear the combo & repopulate - we can't just add the filename if it was a tar.gz
			model = self.cmbProjectType.get_model()
			model.clear()	
			self.get_profiles_list()
			#set the default profile
			self.set_project_type_dropdown(fileBaseName)
				
		file_open.destroy()
			
			
		
def main():
	frm_profiles = frmProfilesManager()
	frm_profiles.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

from classes import messagebox
import os, sys, urllib, uuid
import gtk

# init the foreign language
from language import Language_Init

########################################################################
class OpenShotFile:
	"""The generic file object for OpenShot"""

	#----------------------------------------------------------------------
	def __init__(self, project=None):
		self.project = project
		
		"""Constructor"""
		
		
		# Add language support
		translator = Language_Init.Translator(self.project)
		_ = translator.lang.gettext

		# init the variables for the File Object
		self.name = ""			# short / friendly name of the file
		self.length = 0.0		# length in seconds
		self.videorate = (30,0)	# audio rate or video framerate
		self.file_type = ""		# video, audio, image, image sequence
		self.max_frames = 0.0
		self.fps = 0.0
		self.height = 0
		self.width = 0
		self.label = ""			 # user description of the file
		self.thumb_location = "" # file uri of preview thumbnail
		self.ttl = 1			 # time-to-live - only used for image sequence.  Represents the # of frames per image.
		
		self.unique_id = str(uuid.uuid1())	
		self.parent = None
		self.project = project	  # reference to project
		
		
	def get_thumbnail(self, width, height):
		"""Get and resize the pixbuf thumbnail for a file"""	
		
		# get the thumbnail (or load default)
		try:
			if self.thumb_location:
				pbThumb = gtk.gdk.pixbuf_new_from_file(self.thumb_location)
			else:
				# Load the No Thumbnail Picture
				if self.file_type == "audio":
					pbThumb = gtk.gdk.pixbuf_new_from_file(os.path.join(self.project.IMAGE_DIR, "AudioThumbnail.png"))
				else:
					pbThumb = gtk.gdk.pixbuf_new_from_file(os.path.join(self.project.IMAGE_DIR, "NoThumbnail.png"))
		except:

				# Load the No Thumbnail Picture
				if self.file_type == "audio":
					pbThumb = gtk.gdk.pixbuf_new_from_file(os.path.join(self.project.IMAGE_DIR, "AudioThumbnail.png"))
				else:
					pbThumb = gtk.gdk.pixbuf_new_from_file(os.path.join(self.project.IMAGE_DIR, "NoThumbnail.png"))

		# resize thumbnail
		return pbThumb.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)	
		

########################################################################
class OpenShotFolder:
	"""The generic folder object for OpenShot"""

	#----------------------------------------------------------------------
	def __init__(self, project=None):
		"""Constructor"""
		
		# Init the variables for the Folder Object
		self.name = ""		  # short / friendly name of the folder
		self.location = ""	  # file system location
		self.parent = None
		self.project = project
		
		# init the list of files & folders
		# this list can contain OpenShotFolder or OpenShotFile objects
		# the order of this list determines the order of the tree items
		self.items = []
		
		# this queue holds files that are currently being added. this prevents
		# duplicate files to be added at the same time
		self.queue = []


	#----------------------------------------------------------------------
	def AddFolder(self, folder_name, project=None):
		
		# get a reference to the language translate method
		_ = self.project.translate
		
		"""Add a new folder to the current folder"""
		#does this folder name already exist?
		if self.FindFolder(folder_name):
			messagebox.show(_("OpenShot Error"), _("The folder %s already exists in this project." % folder_name))
			return
		newFolder = OpenShotFolder(project)		
		newFolder.name = folder_name
		
		self.items.append(newFolder)
		
		#set the modified status
		self.project.set_project_modified(is_modified=True, refresh_xml=False, type=_("Added folder"))
		
		self.project.form.refresh()
		

	#----------------------------------------------------------------------
	def AddFile(self, file_name):
		"""Add a new file to the current folder"""
		
		import urllib
		
		# get a reference to the language translate method
		_ = self.project.translate
		
		# clean path to video
		file_name = urllib.unquote(file_name)
		
		# don't add a file that is alrady in this folder (i.e. dupe check)
		if self.file_exists_in_project(file_name):
			return
			
		# check if the path is a 'folder' and not a file
		if os.path.isdir(file_name):
			
			# loop through each sub-file (if any)
			for sub_file in os.listdir(file_name):
				sub_file_path = os.path.join(file_name, sub_file)
				
				# only add files
				if os.path.isfile(sub_file_path):
					
					# don't add a file that is alrady in this folder (i.e. dupe check)
					if self.file_exists_in_project(sub_file_path) == False:

						# inspect the media file and generate it's thumbnail image (if any)
						newFile = self.project.thumbnailer.GetFile(sub_file_path)
						
						# add to internal item collection
						if newFile:
							self.items.append(newFile)

		else:
			# inspect the media file and generate it's thumbnail image (if any)
			newFile = self.project.thumbnailer.GetFile(file_name)
		
			# add to internal item collection
			if newFile:
				self.items.append(newFile)

		# mark project as modified
		self.project.set_project_modified(is_modified=True, refresh_xml=False, type=_("Added file"))
		
		return newFile
	
	#----------------------------------------------------------------------
	def ConvertFileToImages(self, file_name):
		"""Add a new file to the current folder"""
		
		import urllib
		
		# get a reference to the language translate method
		_ = self.project.translate

		# clean path to video
		file_name = urllib.unquote(file_name)

		# check if this file is already in the project
		for item in self.items:
			if file_name in item.name and item.file_type == _("Image Sequence").lower():
				return True
			
		# inspect the media file and generate it's thumbnail image (if any)
		newFile = self.project.thumbnailer.GetFile(file_name, all_frames=True)
		
		# update the location
		if newFile:
			(dirName, fname) = os.path.split(file_name)
			(fileBaseName, fileExtension)=os.path.splitext(fname)
			newFile.name = os.path.join(dirName, fileBaseName, fileBaseName + "_%d.png")
	
			# add to internal item collection
			self.items.append(newFile)

		return


	def file_exists_in_project(self, file_name):
		""" Check if this file exists in this project """
		
		# don't add a file that is alrady in this folder (i.e. dupe check)
		for item in self.items:
			if file_name in item.name:
				return True
		
		# didn't find a match
		return False

	#----------------------------------------------------------------------
	def get_file_path_from_dnd_dropped_uri(self, uri):
		""""""

		path = uri
		#path = urllib.url2pathname(uri) # escape special chars
		path = path.strip('\r\n\x00') # remove \r\n and NULL
		
		if os.name == 'nt':
			path = path.replace("/", "\\")
			path = path.replace("%20", " ")

		# get the path to file
		if path.startswith('file:\\\\\\'): # windows
			path = path[8:] # 8 is len('file:///')
		elif path.startswith('file://'): # nautilus, rox
			path = path[7:] # 7 is len('file://')
		elif path.startswith('file:'): # xffm
			path = path[5:] # 5 is len('file:')
		return path


	def UpdateFileLabel(self, filename, value, refresh_tree=0):
		#this will only be called when the treeview mode is selected, not the thumbview 
		for item in self.items:
			if item.name.endswith(filename):
				item.label = value
				
		if refresh_tree == 1:
			# Update the main form
			self.project.form.refresh()
				
				
	def RemoveFile(self, filename):
		
		# get a reference to the language translate method
		_ = self.project.translate
		
		item = self.FindFile(filename)
		if item:
			# mark project as modified
			self.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Removed file"))
			
			# find clips that have this file object
			for track in self.project.sequences[0].tracks:
				for clip in reversed(track.clips):
					# does clip match file
					if clip.file_object == item:
						# delete clip
						track.clips.remove(clip)
			# remove from file collection
			self.items.remove(item)
		else:
			#is this a folder?
			item = self.FindFolder(filename)
			if item:
				# remove from file collection
				self.items.remove(item)

  
	#----------------------------------------------------------------------
	def FindFile(self, file_name):
		"""Pass the file system location of the item you 
		are looking for and this function will return the 
		reference to the OpenShot File that matches"""
		
		# loop through all files looking for the matching filename
		for file in self.items:
			if isinstance(file, OpenShotFile):
				name = file.name
				
				# check if file name matches this file
				if file_name == name:
					return file

				# split file name (if it's not found)
				(dirName, fileName) = os.path.split(name)
		
				# check if file name matches the basefile name
				if file_name == fileName:
					return file

		
		# No file found
		return None
	
	
	#----------------------------------------------------------------------
	def FindFileByID(self, unique_id):
		"""Pass the file system location of the item you 
		are looking for and this function will return the 
		reference to the OpenShot File that matches"""
		
		# loop through all files looking for the matching filename
		for file in self.items:
			if isinstance(file, OpenShotFile):

				# check if file name matches this file
				if unique_id == file.unique_id:
					return file

		# No file found
		return None

	
	def FindFolder(self, folder_name):
		"""Returns a reference to the OpenShotFolder
		 that matches the folder_name"""
		for folder in self.items:
			if isinstance(folder, OpenShotFolder):
				name = folder.name
				
				if folder_name == name:
					return folder
	
	def ListFolders(self):
		"""Return a list of any folders in the project"""
		
		folders = []
		for item in self.items:
			if isinstance(item, OpenShotFolder):
				folders.append(item.name)
		return folders
	
	def AddParentToFile(self, file_name, folder_name):
		
		# get a reference to the language translate method
		_ = self.project.translate
		
		file = self.FindFile(file_name)
		if file:
			file.parent = folder_name
			
			# mark project as modified
			self.project.set_project_modified(is_modified=True, refresh_xml=False, type=_("Moved to folder"))
			
	def RemoveParent(self, file_name, folder_name):
		
		# get a reference to the language translate method
		_ = self.project.translate
		
		file = self.FindFile(file_name)
		if file:
			print "REMOVE PARENT"
			file.parent = self.project.project_folder
			
			# mark project as modified
			self.project.set_project_modified(is_modified=True, refresh_xml=False, type=_("Removed from folder"))



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys

try:
	import mlt
except ImportError:
	print "*** ERROR: MLT Python bindings failed to import ***"
	

class mlt_profiles:
	
	def __init__(self, project):
		# get the path of OpenShot
		self.project = project
		self.path = self.project.BASE_DIR
		
		# get a list of all mlt profiles
		self.profile_list = []
		
		# get a list of files in the OpenShot /profiles directory
		file_list = os.listdir(self.project.PROFILES_DIR)
		
		for fname in file_list:
			file_name = fname
			file_path = os.path.join(self.project.PROFILES_DIR, file_name)
			
			# load profile object
			p = mlt.Profile(file_path)

			# add to list of profiles
			self.profile_list.append([p.description(), p])
			
		#get any user created profiles
		if os.path.exists(self.project.USER_PROFILES_DIR):
			user_list = os.listdir(self.project.USER_PROFILES_DIR)
			
			for fname in user_list:
				file_name = fname
				file_path = os.path.join(self.project.USER_PROFILES_DIR, file_name)
				
				# load profile object
				p = mlt.Profile(file_path)
	
				# add to list of profiles
				self.profile_list.append([p.description(), p])


	def get_profile_list(self):
		#return a list of mlt.Profile() objects
		#use this syntax to get the name:  self.profile_list[0].description()
		return sorted(self.profile_list)
	
	
	def get_profile(self, profile_name):
		# loop through all profiles
		for fname, p in self.profile_list:
			# compare description to param
			if profile_name == p.description():
				return p
		
		# load default profile object... incase no match was found
		p = mlt.Profile("DV NTSC")
		
		# return the profile
		return p
	
	def profile_exists(self, profile_name):
		# loop through all profiles
		for fname, p in self.profile_list:
			# compare description to param
			if profile_name == p.description():
				return p
			
		#profile doesn't exist
		return False
		
		
def main():
	mlt_profiles1 = mlt_profiles()
	profile_list = mlt_profiles1.get_profile_list()
	mlt_profiles1.get_profile("DV NTSC")

if __name__ == "__main__":
	main()




#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, uuid, locale
import gtk, goocanvas

########################################################################
class transition:
	"""This class represents a media clip on the timeline."""

	#----------------------------------------------------------------------
	def __init__(self, name, position_on_track, length, resource, parent, type="transition", mask_value=50.0):
		"""Constructor"""
		
		# init variables for clip object
		self.name = name
		self.position_on_track = float(position_on_track)  # the time in seconds where the transition starts on the timeline
		self.length = float(length)		# the length in seconds of this transition
		self.resource = resource		# Any grey-scale image, or leave empty for a dissolve
		self.softness = 0.3				# 0.0 = no softness. 1.0 = too soft.	
		self.reverse = False
		self.unique_id = str(uuid.uuid1())
		self.parent = parent			# the sequence
		
		# mask settings
		self.type = type				# transition or mask
		self.mask_value = mask_value	# 0.0 to 1.0
		
		
	def Render(self, exiting_item=None, x_offset = 0):
		
		# get a reference to the 2 main canvas objects & theme
		theme = self.parent.parent.project.theme
		
 		# Get theme settings
 		theme_settings = self.parent.parent.project.theme_settings.settings
		
		# init vars
		pixels_per_second = self.parent.parent.get_pixels_per_second()
		x = float(self.position_on_track * pixels_per_second) + x_offset
		width = self.length * pixels_per_second
		height = theme_settings["transition"]["rectangle"]["h"]
		
		if self.parent:
			# get bottom y of track
			y = self.parent.y_bottom - theme_settings["transition"]["rectangle"]["y"]
		else:
			y = theme_settings["transition"]["rectangle"]["y"]

		# Get root group of the canvas
		canvas_right = self.parent.parent.project.form.MyCanvas
		root_right = canvas_right.get_root_item ()		
		
		if exiting_item == None:
			# Create the Group (for the clip)
			GroupTransition = goocanvas.Group (parent = root_right)
			
			# set the unique ID of the group
			GroupTransition.set_data ("id", self.unique_id)
	
			# Add a translucent blue rectangle
			rec2 = goocanvas.Rect (parent = GroupTransition,
								  x = x,
								  y = y,
								  width = width,
								  height = height,
								  line_width = theme_settings["transition"]["rectangle"]["line_width"],
								  stroke_color_rgba = theme_settings["transition"]["rectangle"]["stroke_color_rgba"],
								  fill_color_rgba = theme_settings["transition"]["rectangle"]["fill_color_rgba"])
			rec2.set_data ("id", "tran_rectangle")
			
			# add text to the transition
			text1 = goocanvas.Text (parent = GroupTransition,
								text = theme_settings["transition"]["transition_name_text"]["font"] % self.name,
								antialias = False,
								x = x + theme_settings["transition"]["transition_name_text"]["x"],
								y = y + theme_settings["transition"]["transition_name_text"]["y"],
								use_markup = True,
								width = width - 8,
								ellipsize = 3)
			text1.set_data ("id", "tran_text")
			
			# hide text (if transition is too small)
			if width <= theme_settings["transition"]["collapse_pixel_threshold_text"]:
				text1.set_properties(visibility = 1)
			else:
				text1.set_properties(visibility = 2)
				
				
			# load clip images
			if self.type == "transition":
				# TRANSITIONS
				if self.reverse:
					# DOWN
					imgTrans = gtk.image_new_from_file("%s/openshot/themes/%s/transition_down.png" % (self.parent.parent.project.form.openshot_path, theme))
				else:
					# UP
					imgTrans = gtk.image_new_from_file("%s/openshot/themes/%s/transition_up.png" % (self.parent.parent.project.form.openshot_path, theme))
			elif self.type == "mask":
					# MASK
					imgTrans = gtk.image_new_from_file("%s/openshot/themes/%s/transition_mask.png" % (self.parent.parent.project.form.openshot_path, theme))				
			
			# create canvas image object
			canvasImageTrans = goocanvas.Image (parent = GroupTransition,
								  pixbuf = imgTrans.get_pixbuf(),
								  x = x + theme_settings["transition"]["thumbnail"]["x"],
								  y = y + theme_settings["transition"]["thumbnail"]["y"])
			canvasImageTrans.lower(text1)
			canvasImageTrans.set_data ("id", "trans_direction")
			
			# hide up/down image (if transition is too small)
			if width <= theme_settings["transition"]["collapse_pixel_threshold_thumbnail"]:
				canvasImageTrans.set_properties(visibility = 1)
			else:
				canvasImageTrans.set_properties(visibility = 2)
				
			
			# connect drag n drop events to the new cavnas group
			GroupTransition.connect ("motion_notify_event", self.on_motion_notify_x)
			GroupTransition.connect ("button_press_event", self.on_button_press_x)
			GroupTransition.connect ("button_release_event", self.on_button_release_x)
		
		else:
			# get existing item
			GroupTransition = exiting_item
			
			# get existing object
			rec2 = self.get_canvas_child(GroupTransition, "tran_rectangle")
			text1 = self.get_canvas_child(GroupTransition, "tran_text")
			canvasImageTrans = self.get_canvas_child(GroupTransition, "trans_direction")
			
			# resize transition
			rec2.set_properties(x = x)
			text1.set_properties(x = x + theme_settings["transition"]["transition_name_text"]["x"])
			canvasImageTrans.set_properties(x = x + theme_settings["transition"]["thumbnail"]["x"])
			rec2.set_properties(width = width)
			text1.set_properties(width = width - theme_settings["transition"]["transition_name_text"]["x"])
			
			# hide text (if transition is too small)
			if width <= theme_settings["transition"]["collapse_pixel_threshold_text"]:
				text1.set_properties(visibility = 1)
			else:
				text1.set_properties(visibility = 2)
				
			# hide up/down image (if transition is too small)
			if width <= theme_settings["transition"]["collapse_pixel_threshold_thumbnail"]:
				canvasImageTrans.set_properties(visibility = 1)
			else:
				canvasImageTrans.set_properties(visibility = 2)

		# return the gooCanvas transition object
		return GroupTransition
	
	
	
	def get_canvas_child(self, group, requested_child_id):
		"""this method loops though the children objects of this group looking 
		for the item with a specfic id."""
		
		for index in range(0, group.get_n_children()):
			child = group.get_child(index)
			child_id = child.get_data ("id")

			if child_id == requested_child_id:
				return child
			
		return None
	
	 
	def on_button_press_x (self, item, target, event):
		""" This method initializes some variables needed for dragging and dropping a clip """
		# raise the group up to the top level
		item.raise_(None)
		
		# set the x and y where the cursor started dragging from
		self.drag_x = event.x
		self.drag_y = event.y

		# only respond to the first mouse button
		if event.button == 1:


			# determine what cursor mode is enable (arrow, razor, snap, etc...)
			(isArrow, isRazor, isSnap, isResize) = self.parent.parent.project.form.get_toolbar_options()
			
			# ARROW MODE
			if isArrow:

				# change the cursor for the drag n drop operation
				fleur = gtk.gdk.Cursor (gtk.gdk.FLEUR)
				canvas = item.get_canvas ()
				self.stored_x = item.get_bounds().x1
				self.stored_y = item.get_bounds().y1
				canvas.pointer_grab (item, gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.BUTTON_RELEASE_MASK, fleur, event.time)
			
			# RESIZE MODE
			elif isResize:
				
				# remember the original length and position before resizing starts
				self.original_length = self.length
				self.original_start_pos = self.position_on_track
				self.stored_x = item.get_bounds().x1
				self.stored_y = item.get_bounds().y1

			# SNAP MODE
			elif isSnap:
				pass

		elif event.button == 3:
			
			# show the track popup menu
			self.parent.parent.project.form.mnuTransition1.showmnu(event, self, item)
 
 
		return True
	
	
	def on_motion_notify_x (self, item, target, event):
		"""this method allows the clip to be dragged and dropped on a track"""	  

 		# Get theme settings
 		theme_settings = self.parent.parent.project.theme_settings.settings

		# get the new x,y coordinates from the mouse
		new_x = event.x
		new_y = event.y	
		
		# get the pixels per second from the parent sequence
		pixels_per_second = self.parent.parent.get_pixels_per_second()
		
		# determine end pixel of sequence
		end_of_timeline = self.parent.parent.length * pixels_per_second
		
		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.parent.parent.project.form.get_toolbar_options()
		
		# ARROW MODE
		if isArrow:
			
			# Move the clip based on the x, y of the mouse
			if (event.state & gtk.gdk.BUTTON1_MASK):

				# don't allow the clip to slide past the beginning of the canvas
				total_x_diff = new_x - self.drag_x 
				total_y_diff = event.y - self.drag_y
				if (item.get_bounds().x1 + total_x_diff < 0):
					total_x_diff = 0 - item.get_bounds().x1
				elif (item.get_bounds().x2 + total_x_diff > end_of_timeline):
					total_x_diff = end_of_timeline - item.get_bounds().x2
				
				# get the track under this transition (the top track)
				drop_track = self.get_valid_drop(item.get_bounds().x1, item.get_bounds().y1)

				# mark project as modified
				self.parent.parent.project.set_project_modified(is_modified=True, refresh_xml=True)
				# move clip
				item.translate (total_x_diff, total_y_diff)



		# RESIZE MODE
		if isResize:
			
			# update cursor
			self.parent.parent.project.form.current_cursor[1] = int(event.x_root)
			self.parent.parent.project.form.current_cursor[2] = int(event.y_root)
			self.parent.parent.project.form.current_cursor[3] = "clip"
			
			if (event.state & gtk.gdk.BUTTON1_MASK):

				# get the direction from the cursor object
				direction = self.parent.parent.project.form.current_cursor[0]
				
				# update transition length text
				text1 = self.get_canvas_child(item, "tran_text")
				text1.set_properties(text = theme_settings["transition"]["transition_name_text"]["font_resize"].replace("|", "\n") % (self.name, locale.format("%.1f", self.length, grouping=False)))
				
				
			else:
				# only calculate LEFT or RIGHT when the mouse is not clicked.  Once the user starts resizing
				# a clip, we want to just use the direction in the cursor object.  In other words, we can't allow
				# the direction to change while we are resizing.
				# determine if user is resizing the LEFT or RIGHT side of the clip
				length = float(self.length)
				
				left_of_clip = self.position_on_track * pixels_per_second
				center_of_clip = left_of_clip + ((length / 2.0) * pixels_per_second)
				right_of_clip = left_of_clip + (length * pixels_per_second)
				
				direction = "left"
				if event.x_root < center_of_clip:
					direction = "left"
				else:
					direction = "right"
				
				# if right, update the cursor the "right grab" icon
				if direction == "right":
					# update cursor variable
					if self.parent.parent.project.form.current_cursor[0] != "right":
	
						# change cursor to "right bar"
						self.parent.parent.project.form.current_cursor[0] = "right"
						self.parent.parent.project.form.MyCanvas.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.RIGHT_SIDE)) 
	
				# if left, update the cursor the "left grab" icon
				if direction == "left":
					# update cursor variable
					if self.parent.parent.project.form.current_cursor[0] != "left":
						
						# change cursor to "left bar"
						self.parent.parent.project.form.current_cursor[0] = "left" 
						self.parent.parent.project.form.MyCanvas.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_SIDE))

			
			# Move the clip based on the x, y of the mouse (if the mouse button is pressed)
			if (event.state & gtk.gdk.BUTTON1_MASK):
				
				# Get side of clip that is being re-sized
				side = self.parent.parent.project.form.current_cursor[0]  # left or right
				
				# Get the x and y difference
				original_end_pos = (self.original_start_pos * pixels_per_second) + (self.original_length * pixels_per_second)
				x_offset = event.x - event.x_root

				if side == "left":
					# calculate new start position and length
					new_position = float(event.x_root) / float(pixels_per_second)
					new_length = float(original_end_pos - event.x_root) / pixels_per_second
					#new_start_time = self.end_time - new_length
						
					# update the properties of this clip
					if new_position >= 0 and new_length >= 0:
						
						# mark project as modified
						self.parent.parent.project.set_project_modified(is_modified=True, refresh_xml=True)
						
						self.position_on_track = new_position
						self.length = new_length
						#self.start_time = new_start_time
						
						# adjust x for windows
						if os.name == 'nt':
							x_offset = x_offset - 2
							
						# re-render this clip to the canvas (passing the canvas group to the RenderClip method)
						self.Render(item, x_offset)

				else:
					# RIGHT side
					# calculate the new clip length
					new_length = (event.x_root - (self.position_on_track * pixels_per_second)) / pixels_per_second
					self.length = new_length

					# update the properties of this clip
					if new_length > 0:
					
						# mark project as modified
						self.parent.parent.project.set_project_modified(is_modified=True, refresh_xml=True)
	
						# re-render this clip to the canvas (passing the canvas group to the RenderClip method)
						self.Render(item, x_offset)

		return True
	
	
	
	def on_button_release_x (self, item, target, event):
		""" This method drops a clip, and snaps the clip to the nearest valid track """

 		# Get theme settings
 		theme_settings = self.parent.parent.project.theme_settings.settings

		# raise the play head to the top
		self.parent.parent.play_head.raise_(None)
		self.parent.parent.play_head_line.raise_(None)

		# get reference to the canvas, and stop dragging the item
		canvas = item.get_canvas()
		canvas.pointer_ungrab (item, event.time)
		
		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.parent.parent.project.form.get_toolbar_options()
		
		if isArrow:
			# get new parent track
			drop_track = self.get_valid_drop(item.get_bounds().x1, item.get_bounds().y1)
			bottom_y = 0
			
			if drop_track == None:
				# keep old parent, if no track found
				drop_track = self.parent
			
			# get bottom y of track
			bottom_y = drop_track.y_bottom - 20
			
			# get the pixels per second from the parent sequence
			pixels_per_second = self.parent.parent.get_pixels_per_second()
			
			# deterime the direction of the drag
			if isSnap:
				distance_from_clip = self.get_snap_difference(self, item)
			else:
				distance_from_clip = 0.0

			# update the position 
			self.position_on_track = float(item.get_bounds().x1 + distance_from_clip) / pixels_per_second
			
			# update the parent (the primary track)
			self.update_parent(drop_track)
			drop_track.reorder_transitions()
			
			# Animate the transition to it's new position
			item.animate(distance_from_clip, bottom_y - item.get_bounds().y1, 1.0, 0.0, False, 200, 4, goocanvas.ANIMATE_FREEZE)
				
			# move the clip object on the timeline to correct position (snapping to the y and x of the track)		
			item.translate (distance_from_clip, bottom_y - item.get_bounds().y1)
			
		# Hide the length label (so it doesn't clutter the timeline)
		text1 = self.get_canvas_child(item, "tran_text")
		text1.set_properties(text = theme_settings["transition"]["transition_name_text"]["font"] % self.name)
			
		# reset the cursor
		self.parent.parent.project.form.MyCanvas.window.set_cursor(None)
		
		# check if clip has been really moved or not
		if self.stored_x == item.get_bounds().x1 and self.stored_y == item.get_bounds().y1 and not isResize:
			type_of_event = None
		elif isResize:
			type_of_event = _("Resized transition")
		else:
			type_of_event = _("Moved transition")
		
		# mark project as modified
		self.parent.parent.project.set_project_modified(is_modified=True, refresh_xml=True, type = type_of_event )
		
		
	def get_valid_drop(self, x1, y1):
		""" A clip must be dropped on a track.  This method returns the track 
		object that is under the clip's current position """

		# loop through each track
		track_counter = 0
		
		for track in self.parent.parent.tracks:
			# get the top y and bottom y of each track
			y_top = track.y_top
			y_bottom = track.y_bottom
			
			# increment track counter
			track_counter = track_counter + 1
			
			# determine if middle of clip is contained inside this track
			if y1 > y_top and y1 < y_bottom and track_counter != len(self.parent.parent.tracks):
				return track
		
		# return false if no valid track found
		return None
	
	
	def update_parent(self, parent_track):
			""" This method updates the x and y settings of the clip in the object model """

			# remove clip from current parent track (if new parent is different)
			if (parent_track != self.parent):
				# remove clip				
				self.parent.transitions.remove(self)
				
				# add to new parent track
				parent_track.transitions.append(self)
				self.parent = parent_track
				
				
	def get_snap_difference(self, clip_object, canvas_item):
		"""Determine the number of pixels to shift this clip to snap to it's 
		closest neighbor clip (if any)"""

		# get the pixels per second from the parent sequence
		pixels_per_second = clip_object.parent.parent.get_pixels_per_second()
		old_x = clip_object.position_on_track * pixels_per_second  # get the old x coordinate of the clip (used to determine direction)

		# get the index of this transition (from the parent collection)
		clip_index = clip_object.parent.transitions.index(clip_object)
		clip_length = clip_object.length * pixels_per_second
		closest_clip = None
		distance_from_clip = 0.0
		distance_from_left_clip = 0.0
		distance_from_right_clip = 0.0
		distance_from_playhead = 0.0
		
		# determine the direction of the drag
		direction = ""
		if canvas_item.get_bounds().x1 < old_x:
			direction = "left"
		else:
			direction = "right"
			
		# get distance from nearby clips (if any)
		nearby_clip_left = self.get_edge_of_clip(canvas_item.get_bounds().x1 / pixels_per_second, "left", self.parent) * pixels_per_second
		distance_from_left_clip = nearby_clip_left - canvas_item.get_bounds().x1
		
		nearby_clip_right = distance_from_right_clip = self.get_edge_of_clip(canvas_item.get_bounds().x2 / pixels_per_second, "right", self.parent) * pixels_per_second
		distance_from_right_clip = (nearby_clip_right + 1) - canvas_item.get_bounds().x2
		
#		# if there is a transition to the left of this one, find the distance to it's edge
#		if (clip_index - 1) >= 0 and (clip_index - 1) < len(clip_object.parent.transitions):
#			closest_clip = clip_object.parent.transitions[clip_index - 1]
#			clip_x = closest_clip.position_on_track * pixels_per_second
#			closest_clip_length = closest_clip.length * pixels_per_second
#			closest_clip_position = clip_x + closest_clip_length
#			distance_from_left_clip = closest_clip_position - canvas_item.get_bounds().x1
#
#		# if there is a transition to the right of this one, find the distance to it's edge
#		if (clip_index + 1) >= 0 and (clip_index + 1) < len(clip_object.parent.transitions):
#			closest_clip = clip_object.parent.transitions[clip_index + 1]
#			closest_clip_position = closest_clip.position_on_track * pixels_per_second
#			distance_from_right_clip = closest_clip_position - (canvas_item.get_bounds().x1 + clip_length)
			
		# distance from the play-head
		playhead_time = clip_object.parent.parent.play_head_position
		playhead_pixels = playhead_time * pixels_per_second
		distance_from_playhead = playhead_pixels - canvas_item.get_bounds().x1
		

		# limit the left /right snapping to 10 pixels
		if distance_from_left_clip > 10 or distance_from_left_clip < -10:
			distance_from_left_clip = 0.0
		if distance_from_right_clip > 10 or distance_from_right_clip < -10:
			distance_from_right_clip = 0.0
		if distance_from_playhead > 10 or distance_from_playhead < -10:
			distance_from_playhead = 0.0

		# determine which direction and what clip to snap to (based on the direction the clip is moving)
		if direction == "left" and distance_from_left_clip != 0:
			distance_from_clip = distance_from_left_clip
		elif direction == "right" and distance_from_right_clip != 0:
			distance_from_clip = distance_from_right_clip
		elif distance_from_left_clip != 0:
			distance_from_clip = distance_from_left_clip
		elif distance_from_right_clip != 0:
			distance_from_clip = distance_from_right_clip
		elif distance_from_playhead != 0:
			distance_from_clip = distance_from_playhead

		# return the # of pixesl to snap the clip
		return distance_from_clip


	def get_edge_of_clip(self, current_position, direction, track): 
		""" Get the position of the closest edge of a track """
		
		if direction == "left":
			# loop through clips on track
			edge = 0.0
			for clip in track.clips:
				# get right edge of clip
				next_edge = float(clip.position_on_track)
				
				# if clip within 1/2 second distance
				if abs(current_position - next_edge) <= 1.0:
					edge = next_edge
					break
				
			# return right edge
			return edge
		
		if direction == "right":
			# loop through clips on track
			edge = 0.0
			for clip in track.clips:
				# get right edge of clip
				next_edge = float(clip.position_on_track) + float(clip.length())
				
				# if clip within 1/2 second distance 
				if abs(current_position - next_edge) <= 1.0:
					edge = next_edge
					break

			# return right edge
			return edge

		# always return something
		return 0.0
		
		
		
		



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import cPickle as pickle
import os
from cStringIO import StringIO as StringIO 
from classes import messagebox, files

def restore_project_state(project_object, state):

	# open the serialized file
	old_form = project_object.form
	old_play_head = project_object.sequences[0].play_head
	old_ruler_time = project_object.sequences[0].ruler_time
	old_thumbnailer = project_object.thumbnailer
	old_play_head_line = project_object.sequences[0].play_head_line
	old_theme = project_object.theme

	# update the form reference on the new project file
	state.seek(0,0)
	project_object = pickle.load(state)

	# re-attach some variables (that aren't pickleable)
	project_object.form = old_form
	project_object.sequences[0].play_head = old_play_head
	project_object.sequences[0].ruler_time = old_ruler_time
	project_object.sequences[0].play_head_line = old_play_head_line
	project_object.thumbnailer = old_thumbnailer
	#project_object.sequences[0].play_head_position = 0.0
	project_object.theme = old_theme
	project_object.mlt_profile = None
	
	# update the thumbnailer's project reference
	project_object.thumbnailer.set_project(project_object)
	
	# update project reference to menus
	project_object.form.mnuTrack1.project = project_object
	project_object.form.mnuClip1.project = project_object
	project_object.form.mnuTransition1.project = project_object
	
	# update the project reference on the form variable
	project_object.form.project = project_object

	# update project references in the menus
	project_object.form.mnuTrack1.project = project_object.form.project
	project_object.form.mnuClip1.project = project_object.form.project
	project_object.form.mnuMarker1.project = project_object.form.project
	project_object.form.mnuTransition1.project = project_object.form.project
	project_object.form.mnuAnimateSubMenu1.project = project_object.form.project
	project_object.form.mnuPositionSubMenu1.project = project_object.form.project
	
	# mark XML as refreshable
	project_object.set_project_modified(is_modified=True, refresh_xml=True)
	
	#check project files still exist in the same location
	missing_files = ""
	items = project_object.project_folder.items
			
	for item in items:
		if isinstance(item, files.OpenShotFile):
			if not os.path.exists(item.name) and "%" not in item.name:
				missing_files += item.name + "\n"
	
	if missing_files:
		messagebox.show("OpenShot", _("The following file(s) no longer exist." + "\n" + "Please restore them or remove them from your project:" + "\n\n" + missing_files))


#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import uuid
import gtk, goocanvas
from classes import clip, files, transition

# init the foreign language
from language import Language_Init

class track:
	"""The track class contains a simple grouping of clips on the same layer (aka track)."""

	#----------------------------------------------------------------------
	def __init__(self, track_name, parent_sequence):
		"""Constructor"""
		
		# Add language support
		translator = Language_Init.Translator(parent_sequence.project)
		_ = translator.lang.gettext
		
		# init variables for sequence
		self.name = track_name
		self.x = 10		 # left x coordinate to start the track
		self.y_top = 0	  # top y coordinate of this track
		self.y_bottom = 0   # bottom y coordinate of this track
		self.parent = parent_sequence   # reference to parent sequence object
		self.play_video = True
		self.play_audio = True
		self.unique_id = str(uuid.uuid1())

		# init the tracks on the sequence
		self.clips = []

		# init transitions
		self.transitions = []
		
		
	
	def AddClip(self, clip_name, color, position_on_track, start_time, end_time, file_object, record_to_history = True):
		# Create new clip object and append to list
		NewClip = clip.clip(clip_name, color, position_on_track, start_time, end_time, self, file_object)
		
		# Adjust default length of images (all images default to 300 seconds.. i.e. 5 minutes)
		# But since nobody wants an image to default to 5 minutes long, we adjust it to the default image length.
		if NewClip.length() == 300.0:
			from windows import preferences
			NewClip.end_time = NewClip.start_time + float(preferences.Settings.general["imported_image_length"])#7.0
		
		# Add clip to track's clip list
		self.clips.append(NewClip)		
		
		# return the new clip
		return NewClip
			
			
	def AddTransition(self, transition_name, position_on_track, length, resource):
		
		# create a new transition object
		new_transition = transition.transition(transition_name, position_on_track, length, resource, self)
		
		# insert new transition
		self.transitions.append(new_transition)	
		
		return new_transition
		 
	
	
	def Render(self):

		# Render this track
		self.RenderTrack()
		
		# loop through each track
		for MyClip in self.clips:
			
			# Render track			
			MyClip.Render()				
		
			
			
			
	def GenerateXML(self, dom, xmlParentNode, fps=None):
		
		playlist = dom.createElement("playlist")
		playlist.setAttribute("id", self.name)
		xmlParentNode.appendChild(playlist)
		
		current_frame = 0
		
		# loop through each track
		for MyClip in self.clips:
			
			# Render track			
			current_frame = MyClip.GenerateXML(dom, playlist, current_frame, fps=fps)


			
	#----------------------------------------------------------------------
	def RenderTrack(self):
		"""This adds a track to the canvas with 3 images: a left, middle, and right"""

		# get a reference to the language translate method
		_ = self.parent.project.translate

		# get the pixels per second from the parent sequence
		pixels_per_second = self.parent.get_pixels_per_second()
		
		# get a reference to the 2 main canvas objects & theme
		theme = self.parent.project.theme
		canvas_left = self.parent.project.form.MyCanvas_Left
		canvas_right = self.parent.project.form.MyCanvas
		
 		# Get theme settings
 		theme_settings = self.parent.project.theme_settings.settings
		
		# get the previous track from the parent sequence (if any)
		previous_track_index = self.parent.tracks.index(self) - 1
		previous_track = None
		previous_y_top = 0
		previous_y_bottom = 0
		
		if (previous_track_index >= 0):
			previous_track = self.parent.tracks[previous_track_index]
			previous_y_top = previous_track.y_top
			previous_y_bottom = previous_track.y_bottom
		
		# set the top coordinate of this track to the bottom coordinate of the last track
		self.y_top = previous_y_bottom + theme_settings["track"]["padding"]

		# Add an item to the goocanvas
		root_left = canvas_left.get_root_item ()
		root_right = canvas_right.get_root_item ()

		# Load all 3 images
		imgTrack_Left = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Left.png" % (self.parent.project.form.openshot_path, theme))
		imgTrack_Middle = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Middle.png" % (self.parent.project.form.openshot_path, theme))
		imgTrack_Right = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Right.png" % (self.parent.project.form.openshot_path, theme))	   		

		# Get Height and Width of Images 
		imgTrack_Left_Height = imgTrack_Left.get_pixbuf().get_height()
		imgTrack_Left_Width = imgTrack_Left.get_pixbuf().get_width()
		imgTrack_Right_Width = imgTrack_Right.get_pixbuf().get_width()		

		# Get Size of Window (to determine how wide the middle image should be streched)
		Size_Of_Middle = int(pixels_per_second * self.parent.length)

		# Resize Middle pixbuf
		middlePixBuf = imgTrack_Middle.get_pixbuf()
		pixbuf_list = self.parent.split_images(middlePixBuf, imgTrack_Left_Height, Size_Of_Middle)	  
		
		# Create Group (for the track)
		GroupTrack = goocanvas.Group (parent = root_right)
		GroupTrack_Left = goocanvas.Group (parent = root_left)
		
		# set the unique ID of the group
		GroupTrack.set_data ("id", self.unique_id)

		# Add Left Image to Group
		image1 = goocanvas.Image (parent = GroupTrack_Left,
								  pixbuf = imgTrack_Left.get_pixbuf(),
								  x = self.x,
								  y = self.y_top)

		# Track the state of the hover over image
		image1.set_data ("id", "normal")
		image1.connect ("button_press_event", self.on_focus_in)
 
		
		# Add Middle Image to Group (this can be multiple image tiled together
		pixbuf_x = 0
		for pixbuf in pixbuf_list:
			# get width of this pixbuf
			pixbuf_width = pixbuf.get_width()
			
			image2 = goocanvas.Image (parent = GroupTrack,
									  pixbuf = pixbuf,
									  x = pixbuf_x,
									  y = self.y_top)  
			
			# increment the x
			pixbuf_x = pixbuf_x + pixbuf_width

		# Add Middle Image to Group
		image3 = goocanvas.Image (parent = GroupTrack,
								  pixbuf = imgTrack_Right.get_pixbuf(),
								  x = Size_Of_Middle - 1,
								  y = self.y_top)
		
		# Add Text to the Track
		text1 = goocanvas.Text (parent = GroupTrack_Left,
								text = theme_settings["track"]["track_name_text"]["font"] % self.name,
								antialias = False,
								use_markup = True,
								x = self.x + theme_settings["track"]["track_name_text"]["x"],
								y = self.y_top + theme_settings["track"]["track_name_text"]["y"])
		
		# Load buttons
		if self.play_video:
			imgTrack_Visible = gtk.image_new_from_file("%s/openshot/themes/%s/visible.png" % (self.parent.project.form.openshot_path, theme))
			imgTrack_Visible.set_tooltip_text(_("Video Visible"))
		else:
			imgTrack_Visible = gtk.image_new_from_file("%s/openshot/themes/%s/not_visible.png" % (self.parent.project.form.openshot_path, theme))
			imgTrack_Visible.set_tooltip_text(_("Video not Visible"))
		if self.play_audio:
			imgTrack_Audio = gtk.image_new_from_file("%s/openshot/themes/%s/speaker.png" % (self.parent.project.form.openshot_path, theme))
			imgTrack_Audio.set_tooltip_text(_("Sound Activated"))
		else:
			imgTrack_Audio = gtk.image_new_from_file("%s/openshot/themes/%s/speaker_mute.png" % (self.parent.project.form.openshot_path, theme))
			imgTrack_Audio.set_tooltip_text(_("Sound Deactivated"))
		
		# Add Visible Image to Group
		image4 = goocanvas.Widget (parent = GroupTrack_Left,
								  widget = imgTrack_Visible,
								  x = self.x + theme_settings["track"]["visible"]["x"],
								  y = self.y_top + theme_settings["track"]["visible"]["y"])  
		
		# Track the state of the hover over image
		image4.connect ("button_press_event", self.on_visible_click)
		
		# Add Audio Image to Group
		image5 = goocanvas.Widget (parent = GroupTrack_Left,
								  widget = imgTrack_Audio,
								  x = self.x + theme_settings["track"]["speaker"]["x"],
								  y = self.y_top + theme_settings["track"]["speaker"]["y"])  
		
		# Track the state of the hover over image
		image5.connect ("button_press_event", self.on_audio_click)
		
		# Increment the Y cooridinate (by the height of the left image)
		self.y_bottom = self.y_top + imgTrack_Left_Height


	def on_visible_click (self, item, target, event):

		# get a reference to the language translate method
		_ = self.parent.project.translate

		# get the parent left group
		parent_group = item.get_parent()

		if self.play_video == True:
			# Load Hover Over
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/not_visible.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			imgTrack_Left_Hover.set_tooltip_text(_("Video not Visible"))
			item.set_properties(widget = imgTrack_Left_Hover)
			
			# update play video variable
			self.play_video = False
			
		else: 
			# Load normal image
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/visible.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			imgTrack_Left_Hover.set_tooltip_text(_("Video Visible"))
			item.set_properties(widget = imgTrack_Left_Hover)

			# update play video variable
			self.play_video = True

		# mark project as modified
		self.parent.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Changed visibility of track"))


		return False
	
	
	def on_audio_click (self, item, target, event):

		# get a reference to the language translate method
		_ = self.parent.project.translate

		# get the parent left group
		parent_group = item.get_parent()

		if self.play_audio == True:
			# Load Hover Over
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/speaker_mute.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			imgTrack_Left_Hover.set_tooltip_text(_("Sound Deactivated"))
			item.set_properties(widget = imgTrack_Left_Hover)
			
			# update play audio variable
			self.play_audio = False
			
		else: 
			# Load normal image
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/speaker.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			imgTrack_Left_Hover.set_tooltip_text(_("Sound Activated"))
			item.set_properties(widget = imgTrack_Left_Hover)
			
			# update play audio variable
			self.play_audio = True

		# mark project as modified
		self.parent.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Changed audio of track"))

		return False

		
	def on_focus_in (self, item, target, event):
		isinstance(item, goocanvas.Image)		
		id = item.get_data ("id")
		
		if event.button == 3:
			# show the track popup menu
			self.parent.project.form.mnuTrack1.showmnu(event, self)

		if id == "normal":
			# Load Hover Over
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Left_Hover.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			item.set_properties(pixbuf = imgTrack_Left_Hover.get_pixbuf())
			item.set_data("id", "hover")
		else: 
			# Load normal image
			imgTrack_Left_Hover = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Left.png" % (self.parent.project.form.openshot_path, self.parent.project.theme))
			item.set_properties(pixbuf = imgTrack_Left_Hover.get_pixbuf())
			item.set_data("id", "normal")


		return False
	
	
	def reorder_clips(self):
		# get a list of all clips on this track
		self.clips.sort(self.compare_clip)
		
		
	def compare_clip(self, MyClip1, MyClip2):
		if MyClip1.position_on_track > MyClip2.position_on_track:
			return 1
		elif MyClip1.position_on_track == MyClip2.position_on_track:
			return 0
		else:
			return -1

		
	def reorder_transitions(self):
		# get a list of all clips on this track
		self.transitions.sort(self.compare_transitions)
		
		
	def compare_transitions(self, MyClip1, MyClip2):
		if MyClip1.position_on_track > MyClip2.position_on_track:
			return 1
		elif MyClip1.position_on_track == MyClip2.position_on_track:
			return 0
		else:
			return -1


#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2010  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys
from xml.dom import minidom
import messagebox

########################################################################
class theme:
	"""This class simplifies the reading of a theme.xml file. """

	#----------------------------------------------------------------------
	def __init__(self, folder_name, project=None):
		"""Constructor for the theme object """

		# Find the Theme XML file
		self.folder_name = folder_name
		self.project = project
		self.theme_path = os.path.join(self.project.THEMES_DIR, folder_name)
		self.theme_xml_path = os.path.join(self.project.THEMES_DIR, folder_name, "theme.xml")
		
		# dictionary to hold theme settings
		self.settings = {}
		
		try:
			# Load the Theme XML file
			self.xmldoc = minidom.parse(self.theme_xml_path)
			
			# get timeline settings
			self.settings["timeline"] = self.get_timeline_settings()
			
			# get track settings
			self.settings["track"] = self.get_track_settings()
			
			# get clip settings
			self.settings["clip"] = self.get_clip_settings()
			
			# get transitions settings
			self.settings["transition"] = self.get_transition_settings()
			
		except:
			# Show friendly error message
			messagebox.show(_("Error!"), _("Unable to load theme XML file: %s.  OpenShot will use the blue_glass XML theme file instead.") % self.theme_xml_path)
			print "Unable to load theme XML file: %s.  OpenShot will use the blue_glass XML theme file instead." % self.theme_xml_path
			
			# clear any half-filled settings dictionary
			self.settings = {}
		
		
	def get_timeline_settings(self):
		
		output = {}
		output["playhead_text"] = {
			"font" : self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('playhead_text')[0].getElementsByTagName('font')[0].firstChild.toxml(),
			"x" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('playhead_text')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('playhead_text')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		output["ruler"] = {
			"height" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('height')[0].firstChild.toxml()),
			
			"playhead" : 
				{
					"x" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead')[0].getElementsByTagName('x')[0].firstChild.toxml()),
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead')[0].getElementsByTagName('y')[0].firstChild.toxml()),
				},
			
			"playhead_line" : 
				{
					"stroke_color" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead_line')[0].getElementsByTagName('stroke_color')[0].firstChild.toxml(), 16),
					"line_width" : float(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead_line')[0].getElementsByTagName('line_width')[0].firstChild.toxml()),
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead_line')[0].getElementsByTagName('y')[0].firstChild.toxml()),
					"length_offset" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('playhead_line')[0].getElementsByTagName('length_offset')[0].firstChild.toxml()),
				},
				
			"time_text" : 
				{
					"font" : self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('time_text')[0].getElementsByTagName('font')[0].firstChild.toxml(),
					"x" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('time_text')[0].getElementsByTagName('x')[0].firstChild.toxml()),
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('time_text')[0].getElementsByTagName('y')[0].firstChild.toxml()),
				},
				
			"small_tick" : 
				{
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('small_tick')[0].getElementsByTagName('y')[0].firstChild.toxml()),
					"h" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('small_tick')[0].getElementsByTagName('h')[0].firstChild.toxml()),
				},
				
			"medium_tick" : 
				{
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('medium_tick')[0].getElementsByTagName('y')[0].firstChild.toxml()),
					"h" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('medium_tick')[0].getElementsByTagName('h')[0].firstChild.toxml()),
				},
				
			"large_tick" : 
				{
					"y" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('large_tick')[0].getElementsByTagName('y')[0].firstChild.toxml()),
					"h" : int(self.xmldoc.getElementsByTagName('timeline')[0].getElementsByTagName('ruler')[0].getElementsByTagName('large_tick')[0].getElementsByTagName('h')[0].firstChild.toxml()),
				},
		}
		
		return output
	
	def get_track_settings(self):
		output = {}
		output["padding"] = int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('padding')[0].firstChild.toxml())
		
		output["track_name_text"] = {
			"font" : self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('track_name_text')[0].getElementsByTagName('font')[0].firstChild.toxml(),
			"x" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('track_name_text')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('track_name_text')[0].getElementsByTagName('y')[0].firstChild.toxml()),
			"w" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('track_name_text')[0].getElementsByTagName('w')[0].firstChild.toxml()),
		}
		
		output["visible"] = {
			"x" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('visible')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('visible')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		output["speaker"] = {
			"x" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('speaker')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('track')[0].getElementsByTagName('speaker')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		return output
	
	def get_clip_settings(self):
		output = {}
		output["collapse_pixel_threshold"] = int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('collapse_pixel_threshold')[0].firstChild.toxml())
		
		output["thumbnail"] = {
			"x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('y')[0].firstChild.toxml()),
			"w" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('w')[0].firstChild.toxml()),
			"h" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('h')[0].firstChild.toxml()),
		}
		
		output["clip_name_text"] = {
			"font" : self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('clip_name_text')[0].getElementsByTagName('font')[0].firstChild.toxml(),
			"font_resize" : self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('clip_name_text')[0].getElementsByTagName('font_resize')[0].firstChild.toxml(),
			"x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('clip_name_text')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('clip_name_text')[0].getElementsByTagName('y')[0].firstChild.toxml()),
			"collapsed_x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('clip_name_text')[0].getElementsByTagName('collapsed_x')[0].firstChild.toxml()),
		}
		
		output["rectangle"] = {
			"stroke_color_rgba" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('stroke_color_rgba')[0].firstChild.toxml(), 16),
			"line_width" : float(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('line_width')[0].firstChild.toxml()),
		}
		
		output["visible"] = {
			"x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('visible')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('visible')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		output["speaker"] = {
			"x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('speaker')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('speaker')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		output["effect"] = {
			"x" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('effect')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('clip')[0].getElementsByTagName('effect')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		return output
	
	def get_transition_settings(self):
		output = {}
		output["collapse_pixel_threshold_text"] = int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('collapse_pixel_threshold_text')[0].firstChild.toxml())
		output["collapse_pixel_threshold_thumbnail"] = int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('collapse_pixel_threshold_thumbnail')[0].firstChild.toxml())

		output["transition_name_text"] = {
			"font" : self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('transition_name_text')[0].getElementsByTagName('font')[0].firstChild.toxml(),
			"font_resize" : self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('transition_name_text')[0].getElementsByTagName('font_resize')[0].firstChild.toxml(),
			"x" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('transition_name_text')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('transition_name_text')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		output["rectangle"] = {
			"line_width" : float(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('line_width')[0].firstChild.toxml()),
			"stroke_color_rgba" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('stroke_color_rgba')[0].firstChild.toxml(), 16),
			"fill_color_rgba" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('fill_color_rgba')[0].firstChild.toxml(), 16),
			"y" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('y')[0].firstChild.toxml()),
			"h" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('rectangle')[0].getElementsByTagName('h')[0].firstChild.toxml()),
		}
		
		output["thumbnail"] = {
			"x" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('x')[0].firstChild.toxml()),
			"y" : int(self.xmldoc.getElementsByTagName('transition')[0].getElementsByTagName('thumbnail')[0].getElementsByTagName('y')[0].firstChild.toxml()),
		}
		
		return output


#def main():
#	mytheme = theme("blue_glass")
#	print mytheme.settings["transition"]["rectangle"]
#
#if __name__ == "__main__":
#	main()


#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk
from classes import messagebox, profiles, project
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp

# init the foreign language
from language import Language_Init


# This form is used to create new projects and save 
# existing projects
class frmNewProject(SimpleGtkBuilderApp):
	
	def __init__(self, mode="", path="NewProject.ui", root="frmNewProject", domain="OpenShot", project=None, **kwargs):
		print "init"
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		self._ = Language_Init.Translator(project).lang.gettext
		_ = self._

		# project instance
		self.project = project
		self.form = project.form

		# check the mode of the form (i.e. new project or save as screen)
		self.mode = mode
		
		# init the project type properties
		self.init_properties()
		
		# init the list of possible project types / profiles
		self.profile_list = profiles.mlt_profiles(self.project).get_profile_list()
		
		# loop through each profile, and add it to the dropdown
		for file_name, p in self.profile_list:
			# append profile to list
			self.cmbProjectType.append_text(p.description())
			
			
		# SET THE SAVE FOLDER LOCATION
		if ".openshot" in self.project.folder:
			# This is the openshot default project (set the folder to 'DESKTOP')
			self.fileProjectFolder.set_current_folder(self.project.DESKTOP)
		
		elif len(self.project.folder) > 0:
			# set default folder (if there is a current folder)
			self.fileProjectFolder.set_current_folder(self.project.folder)
	
			
		if (self.mode == "saveas"):
			# Save project as
			# init the project name
			self.txtProjectName.set_text(self.project.name)
			
			# init the length textbox	
			self.spinProjectLength.set_text(str(self.project.sequences[0].length / 60))
			
			# init the profile
			self.set_dropdown_values(self.project.project_type, self.cmbProjectType)
			
		else:
			# New Project
			# init the project name
			self.txtProjectName.set_text(_("Default Project"))
			
			# init the length textbox	to 10 minutes
			self.spinProjectLength.set_text("10")
			
			# init the profile
			self.set_dropdown_values(self.form.settings.general["default_profile"], self.cmbProjectType)



	def set_dropdown_values(self, value_to_set, combobox):
		
		# get reference to gettext
		_ = self._
		
		model = combobox.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)
			
			# check for the matching value
			if value_to_set == value:			
				
				# set the item as active
				combobox.set_active_iter(iter)
				break
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None and value_to_set not in self.invalid_codecs:
				self.invalid_codecs.append(value_to_set)
				break


	def init_properties(self):
		# get the mlt profile
		localType = self.cmbProjectType.get_active_text()
		p = profiles.mlt_profiles(self.project).get_profile(localType)

		# populate the labels with values
		self.lblHeightValue.set_text(str(p.height()))
		self.lblWidthValue.set_text(str(p.width()))
		self.lblAspectRatioValue.set_text("%s:%s" % (p.display_aspect_num(), p.display_aspect_den()))
		self.lblFrameRateValue.set_text("%.2f" % float(p.fps()))
		self.lblPixelRatioValue.set_text("%s:%s" % (p.sample_aspect_num(), p.sample_aspect_den()))
		
		if p.progressive():
			self.lblProgressiveValue.set_text("Yes")
		else:
			self.lblProgressiveValue.set_text("No")
		

	def new(self):
		print "A new %s has been created" % self.__class__.__name__


	def on_frmNewProject_close(self, widget, *args):
		print "on_frmNewProject_close called with self.%s" % widget.get_name()



	def on_frmNewProject_destroy(self, widget, *args):
		print "on_frmNewProject_destroy called with self.%s" % widget.get_name()

		# Is openshot existing?
		if self.project.form.is_exiting:
			self.project.form.frmMain.destroy()
			

	def on_frmNewProject_response(self, widget, *args):
		print "on_frmNewProject_response called with self.%s" % widget.get_name()


	#def on_fileProjectFolder_selection_changed(self, widget, *args):
	#	print "on_fileProjectFolder_selection_changed called with self.%s" % widget.get_name()


	def on_cmbProjectType_changed(self, widget, *args):
		print "on_cmbProjectType_changed called with self.%s" % widget.get_name()

		# init the project type properties
		self.init_properties()

	def on_btnCancel_clicked(self, widget, *args):
		print "on_btnCancel_clicked called with self.%s" % widget.get_name()

		# close the window		
		self.frmNewProject.destroy()

	def on_btnCreateProject_clicked(self, widget, *args):
		print "on_btnCreateProject_clicked called with self.%s" % widget.get_name()

		localName = str.strip(self.txtProjectName.get_text())
		localFolder = str.strip(self.fileProjectFolder.get_filename())
		localType = self.cmbProjectType.get_active_text()
		localLength = str.strip(self.spinProjectLength.get_text())

		# Validate the the form is valid
		if (len(localName) == 0):
			# Show error message
			messagebox.show(_("Validation Error!"), _("Please enter a valid project name."))

		elif (localType == - 1):
			# Show error message
			messagebox.show(_("Validation Error!"), _("Please enter a valid project type."))

		else:

			
			# check if mode is 'New Project'
			if (self.mode == "new"):
				# Re-init / clear the current project object (to reset all exisint data)
				self.project = project.project()
				self.project.form = self.form
				self.project.form.project = self.project
				
				# clear history (since we are opening a new project)
				self.project.form.history_stack = []
				self.project.form.history_index = -1
				self.project.form.refresh_history()
				self.project.set_project_modified(is_modified=True, refresh_xml=False, type=_("New project"))
			
			# set the project properties
			self.project.name = localName
			self.project.project_type = localType	# set the mlt profile
			self.project.mlt_profile = None			# clear the cached mlt_profile object
			self.project.folder = localFolder
			self.project.sequences[0].length = float(localLength) * 60  # convert to seconds
			self.project.set_project_modified(is_modified=False, refresh_xml=True)

			# stop video
			self.project.form.MyVideo.pause()

			# set the profile settings in the video thread
			self.project.form.MyVideo.set_project(self.project, self.project.form, os.path.join(self.project.USER_DIR, "sequence.mlt"), mode="preview")
			self.project.form.MyVideo.set_profile(localType)
			self.project.form.MyVideo.load_xml()

			# Save the project
			self.project.Save("%s/%s.osp" % (localFolder, localName))

			# Is openshot existing?
			if self.project.form.is_exiting:
				self.project.form.frmMain.destroy()

			# Update the main form
			self.project.form.refresh()

			# close the window
			self.frmNewProject.destroy()


def main():
	frm_new_project = Frmnewproject()
	frm_new_project.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk, gobject, pango
from classes import project, messagebox

# init the foreign language
from language import Language_Init


class OpenShotTree:
	
	def __init__(self, treeview, project):
	
		# Add language support
		self._ = Language_Init.Translator(project).lang.gettext
		_ = self._
	
		# init vars
		self.treeview = treeview
		self.project = project
		
		# create a TreeStore
		self.store = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
	
		# Set the treeview's data model
		self.treeview.set_model(self.store)
		self.treeviewAddGeneralPixbufColumn(self.treeview, _("Thumb"), 0, resizable=False, reorderable=False, project=self.project)
		self.treeviewAddGeneralTextColumn(self.treeview, _("Name"), 1, resizable=False, reorderable=True, editable=False, visible=True, elipses=False, autosize=True, project=self.project)
		self.treeviewAddGeneralTextColumn(self.treeview, "path", 2, resizable=True, reorderable=True, editable=False, visible=False, project=self.project)

		# ADD DEFAULT TRANSITIONS
		file_path = os.path.join(self.project.TRANSITIONS_DIR, "icons", "sand.png")
		
		# get the pixbuf
		pbThumb = gtk.gdk.pixbuf_new_from_file(file_path)

		# resize thumbnail
		# pbThumb = pbThumb.scale_simple(80, 62, gtk.gdk.INTERP_BILINEAR)	
		
		# add transition to tree
		item = self.store.append(None)
		self.store.set_value(item, 0, pbThumb)
		self.store.set_value(item, 1, _("Dissolve"))
		self.store.set_value(item, 2, "")

		# get a list of files in the OpenShot /transitions directory
		file_list = os.listdir(os.path.join(self.project.TRANSITIONS_DIR))

		for fname in sorted(file_list):
			(dirName, file_name) = os.path.split(fname)
			(fileBaseName, fileExtension)=os.path.splitext(file_name)
			file_path = os.path.join(self.project.TRANSITIONS_DIR, "icons", fileBaseName + ".png")
			
			if fileBaseName == "icons":
				# ignore the 'icons' folder
				continue
			
			# get the pixbuf
			pbThumb = gtk.gdk.pixbuf_new_from_file(file_path)

			# resize thumbnail
			#pbThumb = pbThumb.scale_simple(80, 62, gtk.gdk.INTERP_BILINEAR)
			
			# perminately save resized icon
			#pbThumb.save(os.path.join("/home/jonathan/Desktop/thumbnails", fileBaseName + ".png"), "png", {})
			
			# get name of transition
			trans_name = fileBaseName.replace("_", " ").capitalize()

			# add transition to tree
			item = self.store.append(None)
			self.store.set_value(item, 0, pbThumb)
			self.store.set_value(item, 1, _(trans_name))
			self.store.set_value(item, 2, os.path.join(self.project.TRANSITIONS_DIR, file_name))
			
		#get any user created transitions
		if os.path.exists(self.project.USER_TRANSITIONS_DIR):
			user_list = os.listdir(self.project.USER_TRANSITIONS_DIR)
			try:
				for fname in sorted(user_list):
					(dirName, file_name) = os.path.split(fname)
					(fileBaseName, fileExtension)=os.path.splitext(file_name)
					file_path = os.path.join(self.project.USER_TRANSITIONS_DIR, "icons", fileBaseName + ".png")
				
					if fileBaseName == "icons":
						# ignore the 'icons' folder
						continue
				
					# get the pixbuf
					pbThumb = gtk.gdk.pixbuf_new_from_file(file_path)
	
					# get name of transition
					trans_name = fileBaseName.replace("_", " ").capitalize()
	
					# add transition to tree
					item = self.store.append(None)
					self.store.set_value(item, 0, pbThumb)
					self.store.set_value(item, 1, _(trans_name))
					self.store.set_value(item, 2, os.path.join(self.project.USER_TRANSITIONS_DIR, file_name))
			except:
				messagebox.show("Openshot Error!", _("There was an error loading user created transitions."))
			
		
		# connect signals
		self.treeview.connect_after('drag_begin', self.on_treeTransition_drag_begin)
		
	def clear(self):
		self.store.clear()
		
		for column in self.treeview.get_columns():
			self.treeview.remove_column(column)
	
	def on_treeTransition_drag_begin(self, widget, *args):
		context = args[0]
		
		# update drag type
		self.project.form.drag_type = "transition"
	
		# Get the drag icon
		play_image = gtk.image_new_from_file(os.path.join(self.project.THEMES_DIR, self.project.theme, "icons", "plus.png"))
		pixbuf = play_image.get_pixbuf()
		context.set_icon_pixbuf(pixbuf, 15, 10)
		
	
	def treeviewAddGeneralTextColumn(self, treeview, name, pos = 0, resizable=True, reorderable=False, editable=False, visible=True, elipses=False, autosize=False, project=None):
		'''Add a new text column to the model'''
	
		cell = gtk.CellRendererText()
		cell.set_property('editable', editable)
		if (elipses):
			cell.set_property("ellipsize", pango.ELLIPSIZE_END)
		col = gtk.TreeViewColumn(name, cell, markup = pos)
		if (autosize):
			col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
			col.set_expand(False)
		col.set_resizable(resizable)
		col.set_reorderable(reorderable)
		col.set_visible(visible)
		treeview.append_column(col)
		treeview.set_headers_clickable(True)
		
		if (editable):
			model = treeview.get_model()
			cell.connect('edited', self.cell_edited,model, project)
		
		if (reorderable):
			col.set_sort_column_id(pos)
	
		return cell, col
	
	def treeviewAddGeneralPixbufColumn(self, treeview, name, pos = 0, resizable=True, reorderable=False, project=None):
		
		'''Add a new gtk.gdk.Pixbuf column to the model'''
		cell = gtk.CellRendererPixbuf()
		col = gtk.TreeViewColumn(name, cell, pixbuf = pos)
		col.set_resizable(resizable)
		col.set_reorderable(reorderable)
		col.set_alignment(0.0)
		treeview.append_column(col)
		treeview.set_headers_clickable(True)
	
		if (reorderable):
			col.set_sort_column_id(pos)
	
		return cell, col
	
	
	def cell_edited(self, cell, row, new_text, model, project=None):
		
		##Fired when the editable label cell is edited
		#get the row that was edited
		iter = model.get_iter_from_string(row)
		column = cell.get_data(_("Label"))
		#set the edit in the model
		model.set(iter,3,new_text)
		#update the file object with the label edit
		filename = model.get_value(iter, 1)
		project.project_folder.UpdateFileLabel(filename, new_text, 0)





#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.



class timeline:
	"""This class contains methods to simply displaying time codes"""

	#----------------------------------------------------------------------
	def __init__(self):
		"""Constructor"""
		

	def get_friendly_time(self, milli):
		"""Convert milliseconds to a tuple of the common time"""
		sec, milli = divmod(milli, 1000)
		min, sec = divmod(sec, 60)
		hour, min = divmod(min, 60)
		day, hour = divmod(hour, 24)
		week, day = divmod(day, 7)
		
		return (week, day, hour, min, sec, int(milli))


#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk
from classes import messagebox, profiles, project, video
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp
# init the foreign language
from language import Language_Init

class frmTransitionProperties(SimpleGtkBuilderApp):

	def __init__(self, path="TransitionProperties.ui", root="frmTransitionProperties", domain="OpenShot", form=None, project=None, current_transition=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		self._ = _
		
		# add items to direction combo
		options = [_("Transition"), _("Mask")]
		# loop through export to options
		for option in options:
			# append profile to list
			self.cboType.append_text(option)
			
		# add items to direction combo
		options = [_("Up"), _("Down")]
		# loop through export to options
		for option in options:
			# append profile to list
			self.cboDirection.append_text(option)

		self.form = form
		self.project = project
		self.current_transition = current_transition
		self.frmTransitionProperties.show_all()
		
		# init the project type properties
		self.lblName.set_text(self.current_transition.name)
		self.hsSoftness.set_value(self.current_transition.softness * 100.0)
		self.hsThreshold.set_value(self.current_transition.mask_value)
		
		# set the dropdown boxes
		self.set_type_dropdown()
		self.set_direction_dropdown()

		
	def set_type_dropdown(self):
		
		# get correct gettext method
		_ = self._
		
		# get the model and iterator of the project type dropdown box
		model = self.cboType.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)

			# check for the matching project type
			if self.current_transition.type == "mask" and value.lower() == _("Mask").lower():			
				# set the item as active
				self.cboType.set_active_iter(iter)
				break
			
			# check for the matching project type
			if self.current_transition.type == "transition" and value.lower() == _("Transition").lower():			
				# set the item as active
				self.cboType.set_active_iter(iter)
				break
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
		# disable if mask threshold
		if self.current_transition.type == "transition":
			self.hsThreshold.set_sensitive(False)
		else:
			self.hsThreshold.set_sensitive(True)
			
			
	def set_direction_dropdown(self):
		
		# get correct gettext method
		_ = self._
		
		# get the model and iterator of the project type dropdown box
		model = self.cboDirection.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0).lower()

			# check for the matching project type
			if self.current_transition.reverse == False and value == _("Up").lower():			
				# set the item as active
				self.cboDirection.set_active_iter(iter)
				
			# check for the matching project type
			if self.current_transition.reverse == True and value == _("Down").lower():			
				# set the item as active
				self.cboDirection.set_active_iter(iter)
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
		# disable if mask
		if self.current_transition.type == _("Mask").lower():
			self.cboDirection.set_sensitive(False)
		else:
			self.cboDirection.set_sensitive(True)


		
		
	def on_cboType_changed(self, widget, *args):
		print "on_cboType_changed"
		
		# get correct gettext method
		_ = self._
		
		# get new type
		localType = self.cboType.get_active_text()
		
		# disable if mask
		if localType.lower() == _("Mask").lower():
			self.cboDirection.set_sensitive(False)
		else:
			self.cboDirection.set_sensitive(True)
			
		# disable if mask threshold
		if localType.lower() == _("Transition").lower():
			self.hsThreshold.set_sensitive(False)
		else:
			self.hsThreshold.set_sensitive(True)

		
		
	def on_btnCancel_clicked(self, widget, *args):
		print "on_btnCancel_clicked"
		self.frmTransitionProperties.destroy()
		
		
	def on_btnApply_clicked(self, widget, *args):
		print "on_btnApply_clicked"
		
		# get correct gettext method
		_ = self._
		
		# Get settings
		localcboType = self.cboType.get_active_text()
		localcboDirection = self.cboDirection.get_active_text().lower()
		localhsSoftness = self.hsSoftness.get_value()
		localhsThreshold = self.hsThreshold.get_value()
		
		# update transition object
		if localcboType.lower() == _("Mask").lower():
			self.current_transition.type = "mask"
		else:
			self.current_transition.type = "transition"
		if localcboDirection == _("Up").lower():
			self.current_transition.reverse = False
		else:
			self.current_transition.reverse = True
		self.current_transition.softness = float(localhsSoftness) / 100.0
		self.current_transition.mask_value = localhsThreshold
		
		# mark project as modified
		self.project.set_project_modified(is_modified=True, refresh_xml=True)
		
		# Refresh the MLT XML file
		self.project.RefreshXML()
		
		# Refresh form
		self.project.form.refresh()
		
		# close window
		self.frmTransitionProperties.destroy()
		
	
			
def main():
	frmTransitionProperties = frmTransitionProperties()
	frmTransitionProperties.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk
from classes import messagebox, project
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp

# init the foreign language
from language import Language_Init

class frmImportImageSequence(SimpleGtkBuilderApp):

	def __init__(self, path="ImportImageSeq.ui", root="frmImportImageSequence", domain="OpenShot", form=None, project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext

		self.form = form
		self.project = project
		self.frmImportImageSequence.show_all()
		
	def on_btnCancel_clicked(self, widget, *args):
		print "on_btnCancel_clicked"
		self.frmImportImageSequence.destroy()
		
	def on_btnImport1_clicked(self, widget, *args):
		print "on_btnImport1_clicked"
		
		txtFileName1 = str.strip(self.txtFileName.get_text())
		txtFramesPerImage1 = str.strip(self.txtFramesPerImage.get_text())
		folder_location1 = str.strip(self.folder_location.get_filename())
		
		# Validate the the form is valid
		if len(txtFileName1) == 0 or txtFileName1.count("%") == 0:
			# Show error message
			messagebox.show(_("Validation Error!"), _("Please enter a valid file name.  The file name must have a %d (or %04d) where the number section of the file name is supposed to be.  For example:  MyFile_%d.png."))

		elif len(txtFramesPerImage1) == 0:
			# Show error message
			messagebox.show(_("Validation Error!"), _("Please enter an integer in the Frames per Image textbox."))

		else:
		
			# is this file a match?
			wildcard_position = txtFileName1.find("%")
			beginning_of_filename = txtFileName1[0:wildcard_position]
			number_of_matches = 0
			number_of_non_matches = 0
			first_match_path = ""
			
			# loop through a range of files (and be sure at least 2 matches are found)
			for x in range(0, 50000):
				# parse filename
				full_file_name = txtFileName1 % x
				
				# check if file exists
				if os.path.exists(os.path.join(folder_location1, full_file_name)):

					# increment counter 
					number_of_matches = number_of_matches + 1
					number_of_non_matches = 0

					# record first match
					if number_of_matches == 1:
						first_match_path = os.path.join(folder_location1, full_file_name)

				elif number_of_matches > 1:

					# non matching file pattern
					number_of_non_matches = number_of_non_matches + 1
					
					if number_of_non_matches >= 100:
						break

			if number_of_matches <= 1:
				# Show error message
				messagebox.show(_("Validation Error!"), _("At least 2 images must match the file name pattern in the selected folder."))
			
			else: 
							
				# create OpenShotFile (and thumbnail) of the first match
				full_file_path = os.path.join(folder_location1, first_match_path)

				# add file to current project
				f = self.project.project_folder.AddFile(full_file_path)
				
				# Update the file properties (since it's added as an image)
				# We need to make it look like a video
				f.label = _("Image Sequence")
				f.fps = self.project.fps()
				f.max_frames = number_of_matches
				f.ttl = int(txtFramesPerImage1)
				f.length = float(f.max_frames * f.ttl) / float(f.fps)
				f.file_type = "image sequence"
				f.name = os.path.join(folder_location1, txtFileName1)

				# refresh the main form
				self.form.refresh()
				self.frmImportImageSequence.destroy()
		
		
			
def main():
	frmImportImageSequence1 = frmImportImageSequence()
	frmImportImageSequence1.run()

if __name__ == "__main__":
	main()
